package com.jz.dm.mall.moduls.controller.goods;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.jz.common.base.BaseController;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DemandAddReq;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DemandListReq;
import com.jz.dm.mall.moduls.entity.Demand;
import com.jz.dm.mall.moduls.service.DemandService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 用户需求定制申请(TDemand)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:34
 */
@RestController
@RequestMapping("demand")
@Api(tags = "买方--需求管理--controller")
public class DemandController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private DemandService tDemandService;

    /**
     * 条件查询需求列表
     * @param req
     * @return
     */
    @PostMapping("/findList")
    @ApiOperation(value = "我的需求--条件查询需求")
    public Result<IPage<Demand>> findList(@RequestBody DemandListReq req) {
        return tDemandService.findList(req);
    }

    /**
     * 需求发布
     * @param req
     * @return
     */
    @PostMapping("/addDemand")
    @ApiOperation(value = "我的需求--需求发布")
    public Result addDemand(@RequestBody @Valid DemandAddReq req) {
        return tDemandService.addDemand(req);
    }

    /**
     * 查询需求详情
     * @param demandId
     * @return
     */
    @GetMapping("/findByDemand")
    @ApiOperation(value = "我的需求--查询需求详情")
    public Result findByDemand(@RequestParam(name = "demandId") Long demandId) {
        return tDemandService.findByDemand(demandId);
    }
}