package com.jz.dm.mall.moduls.service;

import com.jz.common.exception.ResponseException;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.customer.bean.dto.CustomerDto;
import com.jz.dm.mall.moduls.controller.customer.bean.req.CustomerRequest;
import com.jz.dm.mall.moduls.controller.customer.bean.req.LoginByVSSReq;
import com.jz.dm.mall.moduls.controller.customer.bean.req.LoginRequest;
import com.jz.dm.mall.moduls.entity.MallCustomer;

import javax.servlet.http.HttpServletRequest;

/**
 * 商城用户(TMallCustomer)表服务接口
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
public interface MallCustomerService {

    /**
     * 通过用户账号进行查询
     * @param username
     * @return
     */
    CustomerDto selectByAccount(String username,HttpServletRequest request);

    /**
     * 通过手机号进行查询
     * @param username
     * @return
     */
    CustomerDto selectByPhone(String username, HttpServletRequest request);

    /**
     * 注册账号
     * @param customerRequest
     */
    Result saveCustomer(CustomerRequest customerRequest);

    /**
     * 通过手机号查询是否已注册
     * @param telephone
     * @return
     */
    MallCustomer selectByPhone(String telephone);

    /**
     * 修改密码
     * @param customerId
     * @param password
     */
    void updatePassword(Long customerId, String password);

    /**
     * 通过id查找用户信息
     * @param customerId
     * @return
     */
    MallCustomer selectByUser(Long customerId);

    /**
     * 根据用户id，姐角色id查找菜单
     * @param userId
     * @param roleId
     * @return
     */
    Result getMenu(Long userId, Long roleId);

    /**
     * 登陆
     * @param customer
     * @return
     */
    Result loginByUser(LoginRequest customer);

    /**
     * vss系统登陆
     * @param req
     * @return
     */
    Result loginByVss(LoginByVSSReq req) throws ResponseException;

    /**
     * 获取登陆信息
     * @return
     */
    Result getUserMessage();


//    /**
//     * 忘记密码
//     * @param customer
//     */
//    void upPassword(CustomerRequest customer);
}