package com.jz.dm.mall.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.entity.Department;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.DateUtils;
import com.jz.dm.mall.moduls.controller.company.bean.dto.CityAndAreaDto;
import com.jz.dm.mall.moduls.controller.company.bean.req.CompanyEditReq;
import com.jz.dm.mall.moduls.controller.company.bean.req.CompanyUpdateReq;
import com.jz.dm.mall.moduls.controller.company.bean.dto.CompanyInfoDto;
import com.jz.dm.mall.moduls.entity.*;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.company.bean.req.CompanyAddReq;
import com.jz.dm.mall.moduls.mapper.DepartmentDao;
import com.jz.dm.mall.moduls.mapper.FinanceCustomerAssetsDao;
import com.jz.dm.mall.moduls.mapper.MallCustomerDao;
import com.jz.dm.mall.moduls.service.CompanyAuthService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.mall.moduls.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: CompanyAuthServiceImpl
 * @USER: key
 * @DATE: 2020-12-2/16:35
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("companyAuthService")
@Slf4j
public class CompanyAuthServiceImpl implements CompanyAuthService {
    private static final Logger logger = LoggerFactory.getLogger(CompanyAuthServiceImpl.class);
    @Resource
    private DepartmentDao departmentDao;
    @Resource
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;
    @Resource
    private MallCustomerDao mallCustomerDao;
    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 添加企业认证
     * @param request
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addCompanyData(CompanyAddReq req, HttpServletRequest request) {
        //获取当前用户ID判断当前用户是否已经关联企业
        MallCustomerApiDto currentUser = getUser();
//        // 查询当前用户身份
//        MallUserRole role = mallCustomerDao.findByRoleId(currentUser.getCustomerId());
//        if (role != null && role.getRoleId().toString().split(",").length > 1) {
//            return Result.of_error("已成为卖家/买家！请勿重复操作！");
//        }
//        if (role != null && req.getRoleId().equals(role.getRoleId())) {
//            return Result.of_error("已重复成为卖家/买家!请勿重复操作!");
//        }
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_EXIST);
        }
        MallCustomer mallCustomer = mallCustomerDao.findById(currentUser.getCustomerId());
        if (null == mallCustomer) {
            return Result.of_error(ResultMsg.USER_NOT_EXIST);
        }
        if (null != mallCustomer.getDepartmentId()) {
            return Result.of_error("用户已关联企业,请勿重复操作");
        }

        req.setLoginId(mallCustomer.getCustomerId());
        req.setLoginName(currentUser.getCustomerName());
        if (StringUtils.isNotBlank(req.getDepartmentName()) ||
            StringUtils.isNotBlank(req.getUnifiedCreditCode())) { //企业名称 && 统一社会编码
            Department department = departmentDao.selectDepartmentData(req);
            if (null != department) {
                return Result.of_error("企业用户信息已存在");
            }
        }

        Boolean aBoolean = insetCompany(req);
        if (!aBoolean) {
            throw ResponseException.of_error("添加失败");
        }
        return Result.of_success(ResultMsg.SUCCESS);
    }

    /**
     * 查询企业认证详情
     * @param request
     * @param type
     * @return
     */
    @Override
    public Result selectCompany(String type,HttpServletRequest request) {
        // 查询用户判断用户是否认证
//        MallCustomer currentUser = getCurrentUser(CurrentUser.getCustomerId(request));
        MallCustomerApiDto currentUser = getUser();

        if (null == currentUser) {
            return Result.error("当前用户信息不存在");
        }
        if (StringUtils.equals(type, "STATUS")) {//状态
            String auditStatus = departmentDao.selectCompanyStatus(currentUser.getDepartmentId());
            return Result.of_success(auditStatus);
        } else if (StringUtils.equals(type, "DETAIL")) {//详情
            CompanyInfoDto department = departmentDao.selectCompanyDetail(currentUser.getDepartmentId());
            return Result.of_success(department);
        }
        return Result.of_error("查询企业信息异常");
    }

    /**
     * 获取登录用户信息
     *
     * @param customerId
     * @return
     */
    private MallCustomer getCurrentUser(Long customerId) {
        try {
            MallCustomer mallCustomer = mallCustomerDao.findById(customerId);
            if (null != mallCustomer) {
                return mallCustomer;
            }
        } catch (Exception e) {
            log.error("用户:" + customerId + "信息不存在~~~~~~~~~~");
        }
        return null;
    }

    /**
     * 更新企业信息
     * @param request
     * @param req
     * @return
     */
    @Override
    public Result updateCompanyInfo(CompanyUpdateReq req,HttpServletRequest request) {
//        Long customerId = CurrentUser.getCustomerId(request);
//        MallCustomer currentUser = getCurrentUser(customerId);
        MallCustomerApiDto currentUser = getUser();
        Long customerId = currentUser.getCustomerId();
        if (null == currentUser) {
            return Result.of_error(ResultMsg.USER_NOT_EXIST);
        }
        if (!req.getDepartmentId().equals(customerId)) {
            return Result.of_error("更新企业用户信息与绑定企业信息不一致");
        }
        Department department = new Department();
        BeanUtils.copyProperties(req, department);
        department.setUptTime(new Date());
        department.setUptPerson(req.getLoginName());
        if (departmentDao.updateById(department) != 1) {
            throw ResponseException.of_error("更新企业信息失败");
        }
        return Result.of_success(ResultMsg.UPDATE_SUCCESS);
    }

    /**
     * 校验用户是否已经企业认证
     * @param request
     * @return
     */
    @Override
    public Result checkCompanyInfo(HttpServletRequest request) {
//        MallCustomer currentUser = getCurrentUser(CurrentUser.getCustomerId(request));
        MallCustomerApiDto currentUser = getUser();
        if (null == currentUser) {
            return Result.of_success(ResultMsg.USER_NOT_EXIST);
        }
        CompanyInfoDto companyInfoDto = departmentDao.selectCompanyDetail(currentUser.getDepartmentId());
        if (null != companyInfoDto && "02".equals(companyInfoDto.getAuditStatus())) {
            return Result.of_success("用户已进行企业认证");
        }
        return Result.of_error("用户未进行企业认证");
    }

    /**
     * 获取省份
     * @return
     */
    @Override
    public Result getProvince() {
        Result result = new Result();
        List<Province> list = departmentDao.selectAll();
        if (list.size() == 0) {
            return Result.error("查询失败！");
        }
        result.setData(list);
        return result;
    }

    /**
     * @param provinceCode
     * @param cityCode
     * @return
     */
    @Override
    public Result getCity(String provinceCode,String cityCode) {
        List<CityAndAreaDto>  listProviceName= null;
        if (StringUtils.isNotBlank(provinceCode) && StringUtils.isNotBlank(cityCode)){
            listProviceName=  departmentDao.getAreaList(provinceCode,cityCode);
        }else if (StringUtils.isNotBlank(provinceCode)  && StringUtils.isBlank(cityCode)){
            listProviceName =  departmentDao.getCityList(provinceCode);
        }
        return Result.of_success(listProviceName);
    }

    /**
     * 重新认证企业
     *
     * @param req
     * @return
     */
    @Override
    public Result editCompany(CompanyEditReq req) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("unified_credit_code",req.getUnifiedCreditCode());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        queryWrapper.eq("audit_status", AuditStatusEnum.WTG.getValue());
        Department department = departmentDao.selectOne(queryWrapper);
        if (department != null) {
            department.setAuditStatus(AuditStatusEnum.DSH.getValue());
            if(departmentDao.updateById(department) != 1) {
                return Result.of_error(ResultMsg.UPDATE_FAIL);
            }
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 保存企业信息
     *
     * @param req
     */
    private Boolean insetCompany(CompanyAddReq req) {
        //保存企业信息
        Department departmentInset = new Department();
        departmentInset.setAuditStatus("01");//待审核
        departmentInset.setCreTime(new Date());
        departmentInset.setCrePerson(getUser().getCustomerId() + "");
        BeanUtils.copyProperties(req, departmentInset);
        if (departmentDao.insertDepart(departmentInset) != 1) {
            throw ResponseException.of_error("保存企业信息失败");
        }
        logger.info("###############################企业添加：保存企业信息成功####################################");
        // 更新用户信息
        if (departmentInset.getDepartmentId() != null) {
            MallCustomer mallCustomer = new MallCustomer();
            mallCustomer.setCustomerId(getUser().getCustomerId());
            mallCustomer.setDepartmentId(departmentInset.getDepartmentId());
            mallCustomer.setUptTime(DateUtils.getToday());
            mallCustomer.setUptPerson(getUser().getCustomerId() + "");
            if (mallCustomerDao.updateById(mallCustomer) != 1) {
                throw ResponseException.of_error("更新用户信息失败");
            }
            logger.info("###############################企业添加：更新用户信息成功####################################");
            //初始化用户资产表
            FinanceCustomerAssets finance = new FinanceCustomerAssets();
            finance.setDepartmentId(departmentInset.getDepartmentId());//企业ID
            finance.setCreTime(new Date());
            finance.setCrePerson(getUser().getCustomerId() + "");
            if (financeCustomerAssetsDao.insertFinanceCustome(finance) != 1) {
                throw ResponseException.of_error("初始化用户资产失败");
            }
            logger.info("###############################企业添加：初始化用户资产成功####################################");
//            // 添加角色
//            if (req.getRoleId() != null) {
//                MallUserRole role = new MallUserRole();
//                role.setCustomerId(getUser().getCustomerId());
//                if (2 == req.getRoleId()) {
//                    role.setRoleId(req.getRoleId());
//                }
//                if (3 == req.getRoleId()) {
//                    role.setRoleId(req.getRoleId());
//                }
//                if (0 == mallCustomerDao.addRole(role)) {
//                    throw ResponseException.of_error("赋予角色失败！");
//                }
//                logger.info("###############################企业添加：用户添加角色成功####################################");
//            }
//            return true;
        }
        return false;
    }

    /**
     * 获取当前用户信息
     * @return
     */
    private MallCustomerApiDto getUser() {
        MallCustomerApiDto currentUser = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
        return currentUser;
    }
}
