package com.jz.dm.mall.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.HttpsUtilsTest;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataApiDetailsDto;
import com.jz.dm.mall.moduls.controller.goods.bean.dto.DataGoodsApiDto;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DataApiDetailsReq;
import com.jz.dm.mall.moduls.mapper.DataGoodsApiDao;
import com.jz.dm.mall.moduls.mapper.DataGoodsDao;
import com.jz.dm.mall.moduls.service.DataGoodsApiService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * api商品(TDataGoodsApi)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:31
 */
@Service("dataGoodsApiService")
public class DataGoodsApiServiceImpl implements DataGoodsApiService {
    @Autowired
    private DataGoodsApiDao tDataGoodsApiDao;

    @Autowired
    private DataGoodsDao dataGoodsDao;

    @Value("${gateway.getApiDetail}")
    private String getApiDetail;

    /**
     * 根据主键查询商品api详情
     *
     * @param id
     * @return
     */
//    @Override
//    public Result selectById(Long id) {
//
//        List<DataGoodsApiDto> dataGoodsApiDto = tDataGoodsApiDao.selectById(id);
//        if (dataGoodsApiDto.size() == 0) {
//            return Result.error(StatusCode.ERROR, "查询异常！");
//        }
//        List<DataGoodsApiDto> list1 = new ArrayList<>();
//        List<DataGoodsApiDto> list2 = new ArrayList<>();
//        List<DataGoodsApiDto> list3 = new ArrayList<>();
//        Map<String, Object> map = new HashMap<>();
//
//        for (DataGoodsApiDto goodsApiDto : dataGoodsApiDto) {
//            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("01")) {
//                list1.add(goodsApiDto);
//            }
//            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("02")) {
//                list2.add(goodsApiDto);
//            }
//            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("03")) {
//                list3.add(goodsApiDto);
//            }
//            if (!StringUtils.isEmpty(goodsApiDto.getRequestExample())) {
//                map.put("requestExample", goodsApiDto.getRequestExample()); // 请求示例
//            }
//            if (!StringUtils.isEmpty(goodsApiDto.getReturnDataExample())) {
//                map.put("returnDataExample", goodsApiDto.getReturnDataExample()); // 响应示例
//            }
//        }
//        // 商品详情
//        DataGoodsApiDto goods = dataGoodsApiDto.get(0);
//        DataDetailsDto data = new DataDetailsDto();
//        DecimalFormat df = new DecimalFormat("#0.00");
//        String str = df.format(goods.getDataPrice());
//        BigDecimal bd = new BigDecimal(str).setScale(2, RoundingMode.HALF_UP);
//        data.setDataPrice(bd);
//        BeanUtils.copyProperties(goods, data);
//        map.put("dataInfo", data);
//
//        // 商品参数
//        map.put("pubParam", list1); // 公共参数
//        map.put("reqParam", list2); // 请求参数
//        map.put("resParam", list3); // 响应参数
//
//        Result result = new Result();
//        result.setData(map);
//        return Result.of_success(ResultMsg.SUCCESS, result);
//    }


    /**
     * 根据主键查询商品api详情
     *
     * @param req
     * @return
     */
    @Override
    public Result findByDetails(DataApiDetailsReq req) {

        DataGoodsApiDto dataGoodsApiDto  = dataGoodsDao.selectByDetails(req.getDataGoodsId());
        if (dataGoodsApiDto == null) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        // 获取api详情
        String s = HttpsUtilsTest.submitPost(getApiDetail, JSON.toJSONString(req));
        JSONObject params = JSONObject.parseObject(s);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.of_error("获取api详情失败！");
        }
        DataApiDetailsDto dataDetailsDto = JSONObject.parseObject(params.getString("data"), DataApiDetailsDto.class);
        // 获取请求参数
        List<Map<String, String>> requestParam = getRequestParam(dataDetailsDto.getRequestParam());
        if (requestParam != null) {
            dataGoodsApiDto.setRequestParam(requestParam);
        }
        List<Map<String, String>> responseParam = getRequestParam(dataDetailsDto.getResponseParam());
        if (responseParam != null) {
            dataGoodsApiDto.setResponseParam(responseParam);
        }
        // 赋值商品详情参数
        BeanUtils.copyProperties(dataDetailsDto, dataGoodsApiDto);
        return Result.ok(dataGoodsApiDto);
    }




    /**
     * 获取请求参数
     * @param result
     * @return
     */
    private List<Map<String, String>> getRequestParam(String result) {
        List<Map<String, String>> param = (List<Map<String, String>>) JSONArray.parse(result);
        List<Map<String, String>> resultParam = new ArrayList<>();
        for (Map<String, String> reqMap : param) {
            Map<String, String> map = new HashMap();
            for (Map.Entry entry : reqMap.entrySet()) {
                if (entry.getKey().equals("name")) {
                    map.put("paramsName", entry.getKey().toString());
                }
                if (entry.getKey().equals("type")) {
                    map.put("paramsType", entry.getKey().toString());
                }
                if (entry.getKey().equals("requird")) {
                    map.put("ifRequird",entry.getKey().toString());
                }
                if (entry.getKey().equals("desc")) {
                    map.put("paramsDesc", entry.getKey().toString());
                }
            }
            resultParam.add(map);
        }
        return resultParam;
    }

}