package com.jz.dm.mall.moduls.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.enums.YNEnum;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.entity.DataGoodsCategory;
import com.jz.dm.mall.moduls.entity.SceneCategory;
import com.jz.dm.mall.moduls.mapper.DataGoodsCategoryDao;
import com.jz.dm.mall.moduls.mapper.SceneCategoryDao;
import com.jz.dm.mall.moduls.service.DataGoodsCategoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 数据商品分类(行业)(TDataGoodsCategory)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:32
 */
@Service("dataGoodsCategoryService")
public class DataGoodsCategoryServiceImpl implements DataGoodsCategoryService {
    @Autowired
    private DataGoodsCategoryDao categoryDao;

    @Autowired
    private SceneCategoryDao sceneCategoryDao;


    /**
     * 查询所有的商品分类
     *
     * @return
     */
    @Override
    public Result findCategory() {
        QueryWrapper<DataGoodsCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", "0");
        queryWrapper.eq("if_child", YNEnum.NO.getValue());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        List<DataGoodsCategory> dataGoodsCategories = categoryDao.selectList(queryWrapper);
        if (dataGoodsCategories != null) {
            return Result.ok(dataGoodsCategories);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 查询所有的场景
     *
     * @return
     */
    @Override
    public Result findScene() {
        QueryWrapper<SceneCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        List<SceneCategory> sceneCategories = sceneCategoryDao.selectList(queryWrapper);
        if (sceneCategories != null) {
            return Result.ok(sceneCategories);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }
}