package com.jz.dm.mall.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DemandAddReq;
import com.jz.dm.mall.moduls.controller.goods.bean.request.DemandListReq;
import com.jz.dm.mall.moduls.entity.Demand;
import com.jz.dm.mall.moduls.mapper.DemandDao;
import com.jz.dm.mall.moduls.service.DemandService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

/**
 * 用户需求定制申请(TDemand)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:34
 */
@Service("demandService")
public class DemandServiceImpl implements DemandService {
    @Autowired
    private DemandDao tDemandDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 条件查询需求
     *
     * @param req
     * @return
     */
    @Override
    public Result<IPage<Demand>> findList(DemandListReq req) {
        IPage<Demand> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<Demand> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(req.getSearch())) {
            queryWrapper.and(s -> s.like("demand_name",req.getSearch())
                    .or().like("demand_describe", req.getSearch()));
        }
        if (StringUtils.isNotEmpty(req.getAuditStatus())) {
            queryWrapper.eq("audit_status", req.getAuditStatus());
        }
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        queryWrapper.orderByDesc("cre_time");
        IPage<Demand> list = tDemandDao.selectPage(page, queryWrapper);
        if (list.getTotal() != 0) {
            return Result.of_success(list);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 需求发布
     *
     * @param req
     * @return
     */
    @Override
    public Result addDemand(DemandAddReq req) {
        Demand demand = new Demand();
        demand.setCustomerId(getCustomer().getCustomerId());
        demand.setDemandName(req.getDemandName());
        demand.setDemandDescribe(req.getDemandDescribe());
        demand.setCreTime(DateUtils.getToday());
        demand.setAuditStatus(AuditStatusEnum.DSH.getValue());
        demand.setCrePerson(getCustomer().getCustomerId() + "");
        demand.setDelFlag(DelFlagEnum.NO.getValue());
        if (tDemandDao.insert(demand) != 1) {
            return Result.of_error(ResultMsg.INSERT_SUCCESS);
        }
        return Result.of_success(ResultMsg.INSERT_SUCCESS);
    }

    /**
     * 查询需求详情
     *
     * @param demandId
     * @return
     */
    @Override
    public Result findByDemand(Long demandId) {
        QueryWrapper<Demand> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("demand_id", demandId);
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        Demand demand = tDemandDao.selectOne(queryWrapper);
        if (demand != null) {
            return Result.of_success(demand);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    public MallCustomerApiDto getCustomer() {
        MallCustomerApiDto customer = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
        return customer;
    }

}