package com.jz.dm.mall.moduls.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BasePageBean;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.finance.bean.dto.EveryTradeFlowDto;
import com.jz.dm.mall.moduls.controller.finance.bean.dto.FinanceCashOutDto;
import com.jz.dm.mall.moduls.controller.finance.bean.dto.FinanceCustomerAssetsDto;
import com.jz.dm.mall.moduls.controller.finance.bean.dto.FinanceTradeFlowDto;
import com.jz.dm.mall.moduls.controller.finance.bean.req.EveryTradeFlowRequest;
import com.jz.dm.mall.moduls.controller.finance.bean.req.FinanceCashOutRequest;
import com.jz.dm.mall.moduls.controller.finance.bean.req.FinanceCustomerBalanceRequest;
import com.jz.dm.mall.moduls.entity.FinanceCashOut;
import com.jz.dm.mall.moduls.entity.FinanceCustomerAssets;
import com.jz.dm.mall.moduls.entity.FinanceCustomerBalance;
import com.jz.dm.mall.moduls.mapper.FinanceCustomerAssetsDao;
import com.jz.dm.mall.moduls.mapper.FinanceTradeFlowDao;
import com.jz.dm.mall.moduls.service.FinanceTradeFlowService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 企业客户交易流水(TFinanceTradeFlow)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:39
 */
@Service("financeTradeFlowService")
@Transactional
public class FinanceTradeFlowServiceImpl implements FinanceTradeFlowService {
    private static final Logger logger = LoggerFactory.getLogger(FinanceTradeFlowServiceImpl.class);

    @Autowired
    private FinanceTradeFlowDao financeTradeFlowDao;

    @Autowired
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 充值----获取企业账户余额
     * @param map
     * @return
     * @throws Exception
     */
    @Override
    public FinanceCustomerAssetsDto queryAccountMoneyByMap(Map map) throws Exception {
        return financeTradeFlowDao.queryAccountMoneyByMap(map);
    }

    /**
     * 保存充值金额
     * @param customerBalanceRequest
     * @return
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Boolean uptAccountMoney(FinanceCustomerBalanceRequest customerBalanceRequest) throws Exception {
        if (customerBalanceRequest.getAssetsId() != null && StringUtils.isNotEmpty(customerBalanceRequest.getBalanceMoney().toString())) {
            // 商城企业客户资产表  冻结金额
            FinanceCustomerAssets assets = financeCustomerAssetsDao.findById(customerBalanceRequest.getAssetsId());
            BigDecimal frozenMoney = assets.getFrozenMoney();
            if (frozenMoney.compareTo(BigDecimal.ZERO) > -1) {
                // 添加冻结金额
                BigDecimal add = frozenMoney.add(customerBalanceRequest.getBalanceMoney());
                assets.setFrozenMoney(add);
                assets.setUptPerson(getUser().getCustomerId() + "");
                assets.setUptTime(DateUtils.getToday());
                if (financeCustomerAssetsDao.updateMoney(assets) != 1) {
                    throw ResponseException.of_error("冻结金额添加失败！");
                }
                logger.info("#########################充值金额：冻结金额添加成功！################################");
            }
            //1.充值记录表中加数据
            FinanceCustomerBalance financeCustomerBalance = new FinanceCustomerBalance();
            financeCustomerBalance.setAssetsId(customerBalanceRequest.getAssetsId());//企业资产id
            financeCustomerBalance.setBalanceMoney(customerBalanceRequest.getBalanceMoney()); //充值金额
            financeCustomerBalance.setCrePerson(getUser().getCustomerId()+ "");
            financeCustomerBalance.setBalanceTime(DateUtils.getToday());
            financeCustomerBalance.setCreTime(new Date());
            financeCustomerBalance.setAuditStatus("01"); //待审核
            financeCustomerBalance.setRemark(customerBalanceRequest.getRemark()); //备注
            financeCustomerBalance.setDelFlag("N");
            int len = financeCustomerAssetsDao.addCoustomerCzMoney(financeCustomerBalance);
            return len > 0;
        }
        return false;
    }
    /**
     * 获取可提现金额
     * @param map
     * @return
     * @throws Exception
     */
    @Override
    public FinanceCashOutDto getCashOutAuditStutas(Map map) throws Exception {
        //查询当前企业账户，存在的提现未审核的记录
        BigDecimal auditMoney = new BigDecimal("0");
        List<FinanceCashOutDto> list = financeTradeFlowDao.getCashOutAuditStutas(map);
        if (list.size() > 0 && list != null) {
            for (FinanceCashOutDto dto : list) {
                auditMoney = auditMoney.add(new BigDecimal(dto.getCashOutMoney().toString())); //累加提现待审核 金额
            }
        }
        FinanceCashOutDto returnDto = new FinanceCashOutDto();
        returnDto.setCashOutMoney(auditMoney);
        return returnDto;
    }
    /**
     * 保存提现金额
     * @param financeCashOutRequest
     * @return
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result<Object> addCashOutMoney(FinanceCashOutRequest financeCashOutRequest) throws Exception {
        Result<Object> result = new Result<>();
        Long assetsId = financeCashOutRequest.getAssetsId(); // 企业资产id
        BigDecimal cashOutMoney = new BigDecimal(financeCashOutRequest.getCashOutMoney().toString()); //提现金额

        FinanceCashOut financeCashOut = new FinanceCashOut();
        BeanUtils.copyProperties(financeCashOutRequest, financeCashOut);
        financeCashOut.setCreTime(new Date()); //创建时间
        financeCashOut.setCashOutTime(new Date()); //提现时间
        financeCashOut.setAuditStatus(AuditStatusEnum.DSH.getValue()); //待审核
        financeCashOut.setDelFlag("N");
        financeCashOut.setCrePerson(getUser().getCustomerId()+ "");
        //添加一条提现记录
        int len = financeTradeFlowDao.addCashOutMoney(financeCashOut);

        if (len == 0) {
            result.setMessage("提交失败！");
            result.setCode(500);
            return result;
        }
        // 查询 当前企业 可用余额 冻结金额
        Map map = new HashMap();
        map.put("assetsId", assetsId);

        FinanceCustomerAssetsDto dto = financeTradeFlowDao.queryAccountMoneyByMap(map);
        BigDecimal useMoney = new BigDecimal(dto.getUseMoney().toString()); //可用金额
        if (useMoney != null && cashOutMoney != null) {
            if (useMoney.compareTo(cashOutMoney) > -1) { //可用大于等于提现
                useMoney = useMoney.subtract(cashOutMoney); //可用 减去 提现
                BigDecimal djMoney = new BigDecimal(dto.getFrozenMoney().toString()); //冻结金额
                // 冻结金额必须大于等于0
                if (djMoney.compareTo(BigDecimal.ZERO) > -1) {
                    djMoney = djMoney.add(cashOutMoney); //累计冻结金额

                    //修改企业资产 金额
                    FinanceCustomerAssets assets = new FinanceCustomerAssets();
                    assets.setUptTime(new Date());
                    assets.setUptPerson(getUser().getCustomerId() + "");
                    assets.setUseMoney(useMoney);
                    assets.setFrozenMoney(djMoney);
                    assets.setTotalMoney(useMoney);
                    assets.setAssetsId(assetsId);
                    int lens = financeCustomerAssetsDao.uptAccountMoney(assets);
                    if (lens == 0) {
                        result.setMessage("提交失败！");
                        result.setCode(500);
                        return result;
                    }
                } else {
                    result.setMessage("提现金额必须小于等于可用金额！");
                    result.setCode(500);
                    return result;
                }
            }
            return Result.error("金额小于0!");
        }
        return result;
    }
    /**
     * 收支情况-分页查询
     * @param pageBean
     * @param req
     * @return
     * @throws Exception
     */
    @Override
    public PageInfoResponse<FinanceTradeFlowDto> findListScTradeFlow(BasePageBean pageBean, HttpServletRequest req) throws Exception {
        PageInfoResponse<FinanceTradeFlowDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize());
        Map param = new HashMap();
        //param.put("assetsId",assetsId); //企业资产id
        List<FinanceTradeFlowDto> list = financeTradeFlowDao.findListScTradeFlow(param);
        PageInfo<FinanceTradeFlowDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }
    /**
     * 点击查看--每天--收支情况（分页查询）
     * @param everyTradeFlowRequest
     * @param req
     * @return
     * @throws Exception
     */
    @Override
    public PageInfoResponse<EveryTradeFlowDto> findListEveryDayScInfo(EveryTradeFlowRequest everyTradeFlowRequest, HttpServletRequest req) throws Exception {
        PageInfoResponse<EveryTradeFlowDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(everyTradeFlowRequest.getPageNum(), everyTradeFlowRequest.getPageSize());
        Map param = new HashMap();
        //商品分类id（行业）
        if (!StringUtils.isEmpty(everyTradeFlowRequest.getCreTime())) {
            param.put("creTime", everyTradeFlowRequest.getCreTime());
        }
        //param.put("assetsId",assetsId); //企业资产id
        List<EveryTradeFlowDto> list = financeTradeFlowDao.findListEveryDayScInfo(param);
        PageInfo<EveryTradeFlowDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 获取当前商城用户
     * @return
     */
    public MallCustomerApiDto getUser() {
        MallCustomerApiDto user = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
        return user;
    }
}