package com.jz.dm.mall.moduls.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.entity.Department;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.Base64Util;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.RedisUtil;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.customer.bean.dto.CustomerDto;
import com.jz.dm.mall.moduls.controller.customer.bean.req.CustomerRequest;
import com.jz.dm.mall.moduls.controller.customer.bean.req.LoginByVSSReq;
import com.jz.dm.mall.moduls.controller.customer.bean.req.LoginRequest;
import com.jz.dm.mall.moduls.entity.FinanceCustomerAssets;
import com.jz.dm.mall.moduls.entity.MallCustomer;
import com.jz.dm.mall.moduls.entity.MallMenu;
import com.jz.dm.mall.moduls.mapper.DepartmentDao;
import com.jz.dm.mall.moduls.mapper.FinanceCustomerAssetsDao;
import com.jz.dm.mall.moduls.mapper.MallCustomerDao;
import com.jz.dm.mall.moduls.service.MallCustomerService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * 商城用户(TMallCustomer)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@Service("mallCustomerService")
public class MallCustomerServiceImpl implements MallCustomerService {
    private static final Logger logger = LoggerFactory.getLogger(MallCustomerServiceImpl.class);
    @Autowired
    private MallCustomerDao tMallCustomerDao;

    @Autowired
    private DepartmentDao departmentDao;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private FinanceCustomerAssetsDao assetsDao;


    /**
     * 通过用户账号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByAccount(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByAccount(username);
        return mallCustomer;
    }

    /**
     * 通过手机号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByPhone(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByPhone(username);
        return mallCustomer;
    }

    /**
     * 注册账号
     *
     * @param
     */
    @Override
    public Result saveCustomer(CustomerRequest customerRequest) {
        // 获取验证码
        String vailCode = customerRequest.getVailCode();
        String telephone = customerRequest.getTelephone();
        String username = customerRequest.getUsername();

        Map mapUser = new HashMap();
        if (StringUtils.isNotEmpty(customerRequest.getUsername())) {
            mapUser.put("customerAccount", customerRequest.getUsername());
        }
        MallCustomerApiDto customerUser = tMallCustomerDao.selectByUser(mapUser);
        if (customerUser != null) {
            return Result.of_error("用户已注册！");
        }
        Map mapPhone = new HashMap();
        if (StringUtils.isNotEmpty(customerRequest.getTelephone())) {
            mapPhone.put("customerPhone", customerRequest.getTelephone());
        }
        MallCustomerApiDto customerPhone = tMallCustomerDao.selectByUser(mapPhone);
        if (customerPhone != null) {
            return Result.of_error("手机号已注册！");
        }

        // 从redis获取验证码
//        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
//        Integer codeInRedis = (Integer) redisTemplate.opsForValue().get(key);
//        if (codeInRedis == null) {
//            return Result.of_error("验证码为空！请重新输入！");
//        }
        if (vailCode.equals("115522")) {
            MallCustomer customer = new MallCustomer();
            customer.setCustomerAccount(username);
            customer.setPassword(Base64Util.encode(customerRequest.getPassword()));
            customer.setCustomerPhone(telephone);
            customer.setCreTime(DateUtils.getToday());
            customer.setCustomerSource("01");
            customer.setRegisterTime(DateUtils.getToday());
            tMallCustomerDao.saveCustomer(customer);
            return Result.of_success("注册成功！");
        }
        return Result.of_error("注册失败！");
    }

    /**
     * 通过手机号查询是否已注册
     *
     * @param telephone@return
     */
    @Override
    public MallCustomer selectByPhone(String telephone) {
        MallCustomer mallCustomer = tMallCustomerDao.selectByPhoneExist(telephone);
        if (mallCustomer != null) {
            return mallCustomer;
        }
        return null;
    }

    /**
     * 修改密码
     *
     * @param customerId
     * @param password
     */
    @Override
    public void updatePassword(Long customerId, String password) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        if (mallCustomer != null) {
            MallCustomer customer = new MallCustomer();
            mallCustomer.setPassword(password);
            mallCustomer.setUptTime(DateUtils.getToday());
            mallCustomer.setUptPerson(mallCustomer.getCustomerId() + "");
            mallCustomer.setCustomerId(mallCustomer.getCustomerId());
            tMallCustomerDao.updatePassword(customer);
        }
    }

    /**
     * 通过id查找用户信息
     *
     * @param customerId
     * @return
     */
    @Override
    public MallCustomer selectByUser(Long customerId) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        return mallCustomer;
    }

    /**
     * 根据用户id，角色id查找菜单
     *
     * @param userId
     * @param roleId
     * @return
     */
    @Override
    public Result getMenu(Long userId, Long roleId) {

        //查询用户拥有的角色
        List<MallMenu> entities = tMallCustomerDao.getMenu(userId, roleId);

        // 找到所有的一级分类
        List<MallMenu> level1Menus = entities.stream().filter(sysMenuEntity ->
                sysMenuEntity.getParentId() == 0 && sysMenuEntity.getIfChild().equals("N")
        ).map((menu) -> {
            menu.setChildren(getChildrens(menu, entities));
            return menu;
        }).collect(Collectors.toList());
        return Result.of_success(ResultMsg.SUCCESS, level1Menus);
    }

    /**
     * 登陆
     *
     * @param customer
     * @return
     */
    @Override
    public Result loginByUser(LoginRequest customer) {
        // 手机登陆
        String ph = "^[1][34578]\\d{9}$";
        Map map = new HashMap();
        if (customer.getUsername().matches(ph) && StringUtils.isNotEmpty(customer.getUsername())) {
            map = new HashMap();
            map.put("customerPhone", customer.getUsername());
            MallCustomerApiDto mallCustomer = tMallCustomerDao.selectByUser(map);
            if (mallCustomer.getCustomerId() == null) {
                return Result.of_error("用户不存在，请注册！");
            }
            if (customer.getPassword().equals(Base64Util.decode(mallCustomer.getPassword()))) {
                redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomer, 3, TimeUnit.DAYS);

                return Result.of_success(ResultMsg.LOGIN_SUCCESS, mallCustomer);
            }
            return Result.of_error(ResultMsg.LOGIN_ERROR);
        }
        map = new HashMap();
        map.put("customerAccount", customer.getUsername());
        MallCustomerApiDto mallCustomer = tMallCustomerDao.selectByUser(map);
        if (mallCustomer == null) {
            return Result.of_error("用户不存在，请注册！");
        }
        if (Base64Util.decode(mallCustomer.getPassword()).equals(customer.getPassword())) {
//            // 查询当前用户有无角色
//            if (StringUtils.isNotBlank(mallCustomer.getRoleId())) {
//                MallCustomerApiDto mallCustomerApiDto = new MallCustomerApiDto();
//                BeanUtils.copyProperties(mallCustomer,mallCustomerApiDto);
//                redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
//            }
            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomer, 3, TimeUnit.DAYS);

            return Result.of_success(ResultMsg.LOGIN_SUCCESS, mallCustomer);
        }
        return Result.of_error(ResultMsg.LOGIN_ERROR);
    }

    /**
     * vss系统登陆
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result loginByVss(LoginByVSSReq req) throws ResponseException {
        MallCustomerApiDto mallCustomerApiDto = tMallCustomerDao.selectByVSS(req);
        // 如果不为空，则登陆
        if (mallCustomerApiDto != null) {
            // 企业名称不同，覆盖信息
            if (!mallCustomerApiDto.getDepartmentName().equals(req.getDepartmentName())) {
                // 更新企业名称
                Department department = new Department();
                department.setDepartmentId(mallCustomerApiDto.getDepartmentId());
                department.setDepartmentName(req.getDepartmentName());
                department.setUptPerson(mallCustomerApiDto.getCustomerId() + "");
                department.setDepartmentSource("02");
                department.setUptTime(DateUtils.getToday());
                department.setDelFlag(DelFlagEnum.NO.getValue());
                if (departmentDao.updateById(department) != 1) {
                    throw ResponseException.of_error(ResultMsg.UPDATE_FAIL);
                }
                logger.info("========================vss账号登陆：企业信息更新成功！================================");
            }
            // 如果供应商编号不一致
            if (!mallCustomerApiDto.getVendorId().equals(req.getVendorId())) {
                // 关系供应商编号
                MallCustomer mallCustomer = new MallCustomer();
                mallCustomer.setCustomerId(mallCustomerApiDto.getCustomerId());
                mallCustomer.setCustomerSource("02");
                mallCustomer.setVendorId(req.getVendorId());
                mallCustomer.setUptPerson(mallCustomerApiDto.getCustomerId() + "");
                mallCustomer.setUptTime(DateUtils.getToday());
                if (tMallCustomerDao.updateById(mallCustomer) != 1) {
                    throw ResponseException.of_error(ResultMsg.UPDATE_FAIL);
                }
                logger.info("========================vss账号登陆：用户信息更新成功！================================");
            }
            //登陆
            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
            return Result.ok(mallCustomerApiDto);
        }
        // 如果为null
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("unified_credit_code", req.getUnifiedCreditCode());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        Department department = departmentDao.selectOne(queryWrapper);
        if (department == null) {
            department = insertDepartment(req);
            // 新增资产
            insertAssets(department);
        }
        // 查询资产
        QueryWrapper<FinanceCustomerAssets> wrapper = new QueryWrapper<>();
        wrapper.eq("department_id", department.getDepartmentId());
        wrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        FinanceCustomerAssets assets = assetsDao.selectOne(wrapper);
        if (assets == null) {
            return Result.of_error("资产账号为空！");
        }
        // 查询账号
        QueryWrapper<MallCustomer> query = new QueryWrapper<>();
        query.eq("customer_account", req.getCustomerAccount());
        query.eq("customer_source", "02");
        query.eq("del_flag", DelFlagEnum.NO.getValue());
        MallCustomer mallCustomer = tMallCustomerDao.selectOne(query);
        if (mallCustomer == null) {
            // 新增账号
            mallCustomer = insetCustomer(department, req);
        }
        MallCustomerApiDto apiDto = new MallCustomerApiDto();
        apiDto.setCustomerId(mallCustomer.getCustomerId());
        apiDto.setAssetsId(assets.getAssetsId() + "");
        apiDto.setDepartmentId(department.getDepartmentId());
        apiDto.setCustomerAccount(mallCustomer.getCustomerAccount());
        apiDto.setDepartmentName(department.getDepartmentName());
        apiDto.setVendorId(mallCustomer.getVendorId());
        redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, apiDto, 3, TimeUnit.DAYS);
        return Result.ok(apiDto);
    }

    /**
     * 新增企业
     */
    private Department insertDepartment(LoginByVSSReq req) {
        // 新增企业
        Department department = new Department();
        department.setDepartmentName(req.getDepartmentName());
        department.setUnifiedCreditCode(req.getUnifiedCreditCode());
        department.setCreTime(DateUtils.getToday());
        department.setAuditTime(department.getCreTime());
        department.setDepartmentSource("02");
        department.setAuditStatus(AuditStatusEnum.YSH.getValue());
        if (departmentDao.insert(department) != 1) {
            throw ResponseException.of_error(ResultMsg.INSERT_FAIL);
        }
        logger.info("========================vss账号登陆：新增企业信息成功！================================");
        return department;
    }

    /**
     * 新增资产
     */
    private Result insertAssets(Department department) {
        FinanceCustomerAssets assets = new FinanceCustomerAssets();
        assets.setDepartmentId(department.getDepartmentId());
        assets.setCreTime(DateUtils.getToday());
        assets.setDelFlag(DelFlagEnum.NO.getValue());
        if (assetsDao.insert(assets) != 1) {
            throw ResponseException.of_error(ResultMsg.INSERT_FAIL);
        }
        logger.info("========================vss账号登陆：新增企业资产信息成功！================================");
        return Result.ok();
    }

    /**
     * 新增账号信息
     */
    private MallCustomer insetCustomer(Department department, LoginByVSSReq req) {
        MallCustomer mallCustomer = new MallCustomer();
        mallCustomer.setCustomerAccount(req.getCustomerAccount());
        mallCustomer.setDepartmentId(department.getDepartmentId());
        mallCustomer.setCustomerSource("02");
        mallCustomer.setVendorId(req.getVendorId());
        mallCustomer.setCreTime(DateUtils.getToday());
        mallCustomer.setCrePerson(mallCustomer.getCustomerId() + "");
        mallCustomer.setRegisterTime(DateUtils.getToday());
        mallCustomer.setDelFlag(DelFlagEnum.NO.getValue());
        if (tMallCustomerDao.insert(mallCustomer) !=1){
            throw ResponseException.of_error(ResultMsg.INSERT_FAIL);
        }
        logger.info("========================vss账号登陆：新增用户信息成功！================================");
        return mallCustomer;
    }

    /**
     * 获取登陆信息
     *
     * @return
     */
    @Override
    public Result getUserMessage() {
        MallCustomerApiDto customer = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
        if (customer != null) {
            return Result.ok(customer);
        }
        return Result.of_error(ResultMsg.LOGIN_ERROR);
    }


    private List<MallMenu> getChildrens(MallMenu sysMenu, List<MallMenu> all) {

        List<MallMenu> children = all.stream().filter(sysMenuEntity -> {
            return sysMenuEntity.getParentId() == sysMenu.getMenuId() && sysMenuEntity.getIfChild().equals("Y");
        }).map(sysMenuEntity -> {
            //1、找到子菜单,同时递归调用
            sysMenuEntity.setChildren(getChildrens(sysMenuEntity, all));
            return sysMenuEntity;
        }).collect(Collectors.toList());

        return children;
    }


}