package com.jz.dm.mall.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.HttpsUtilsTest;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.customer.bean.dto.PurchaserUserCenterDto;
import com.jz.dm.mall.moduls.controller.customer.bean.req.PurchaserUserCenterReq;
import com.jz.dm.mall.moduls.controller.order.bean.dto.*;
import com.jz.dm.mall.moduls.controller.order.bean.req.LogInfoListReq;
import com.jz.dm.mall.moduls.controller.order.bean.req.OrderRequest;
import com.jz.dm.mall.moduls.controller.order.bean.req.SaltResetReq;
import com.jz.dm.mall.moduls.entity.Order;
import com.jz.dm.mall.moduls.mapper.DepartmentDao;
import com.jz.dm.mall.moduls.mapper.OrderDao;
import com.jz.dm.mall.moduls.service.OrderService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 订单表(TOrder)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@Service("orderService")
@Transactional
public class OrderServiceImpl implements OrderService {

    @Autowired
    private OrderDao orderDao;

    //@Autowired
    //private HttpsUtils httpsUtils;

    @Autowired
    private DepartmentDao departmentDao;

    @Autowired
    private RedisTemplate redisTemplate;



    @Value("${gateway.gatewayOrderLogDetails}")
    private String gatewayOrderLogDetails;

    @Value("${gateway.gatewayOrderLogNumError}")
    private String gatewayOrderLogNumError;

    @Value("${gateway.gatewayUpSalt}")
    private String gatewayUpSalt;
    /**
     * 订单管理列表（分页查询）
     *
     * @author Bellamy
     */
    @Override
    public PageInfoResponse<OrderDto> findList(OrderRequest order, HttpServletRequest req) throws Exception {
        PageInfoResponse<OrderDto> pageInfoResponse = new PageInfoResponse<>();

        Map<String, Object> param = new HashMap<>();
        //数据编号
        if (StringUtils.isNotEmpty(order.getOrderNumber())) {
            param.put("orderNumber", order.getOrderNumber());
        }
        //购买（订单）方式
        if (StringUtils.isNotEmpty(order.getPurchaseMethod())) {
            param.put("purchaseMethod", order.getPurchaseMethod());
        }
        //数据类型
        if (StringUtils.isNotEmpty(order.getDataType())) {
            param.put("dataType", order.getDataType());
        }
        //数据商品名称
        if (!StringUtils.isEmpty(order.getDataName())) {
            param.put("dataName", order.getDataName());
        }
        //商品分类id（行业）
        if (!StringUtils.isEmpty(order.getCategoryId())) {
            param.put("categoryId", order.getCategoryId());
        }

        PageHelper.startPage(order.getPageNum(), order.getPageSize());
        List<OrderDto> list = orderDao.findList(param);
        PageInfo<OrderDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }
    /**
     * 根据订单主键查询，订单详情
     * @param orderId
     * @return
     * @throws Exception
     */
    @Override
    public OrderDto getOrderDetail(String orderId) throws Exception {
        OrderDto orderDto = null;
        if (StringUtils.isNotEmpty(orderId)) {
            orderDto = orderDao.getOrderDetail(orderId);
            if (orderDto != null) {
                Date endTime = orderDto.getInvalidTime();
                String indate = DateUtils.getTime(endTime, DateUtils.getToday());
                orderDto.setIndate(indate);
            }
        }
        return orderDto;
    }
    /**
     * 根据订单主键查询，订单详情-接口文档
     * @param params
     * @return
     * @throws Exception
     */
    @Override
    public Map getApiInterfaceDetail(Map params) throws Exception {
        Map returnMap = new HashMap();
        DataGoodsApiDto dataGoodsApi = new DataGoodsApiDto(); //接口文档信息
        List<DataGoodsApiParamsDto> goodsApiParamsLst = new ArrayList<>(); //接口参数信息

        if (params.size() > 0) {
            List<DataGoodsApiDto> dataGoodsApiDto = orderDao.getApiInterface(params);//接口文档信息
            dataGoodsApi = dataGoodsApiDto.get(0);
            Long goodsApiId = dataGoodsApi.getGoodsApi();
            goodsApiParamsLst = orderDao.getApiParamsInfo(goodsApiId);//接口参数信息
        }
        returnMap.put("dataGoodsApi", dataGoodsApi);
        returnMap.put("goodsApiParamsLst", goodsApiParamsLst);
        return returnMap;
    }


    /**
     * 订单管理列表（分页查询）--卖方
     * @param req
     * @param httpServletRequest
     * @return
     */
    @Override
    public Result<IPage<OrderBySellerDto>> queryPageListBySeller(OrderRequest req, HttpServletRequest httpServletRequest) {
        IPage<OrderBySellerDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<OrderBySellerDto> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getSearch())) {
            query.and(wrapper -> wrapper.like("t2.data_name", req.getSearch())
                    .or().like("t4.department_name", req.getSearch()));
        }
        if (StringUtils.isNotBlank(req.getPurchaseMethod())) {
            query.eq("t.purchaseMethod", req.getPurchaseMethod());
        }

        if (StringUtils.isNotBlank(req.getOrderTime())) {
            String startTime = req.getOrderTime().substring(0, 10) + " 00:00:00";
            String endTime = req.getOrderTime().substring(11, 21) + " 23:59:59";
            query.between("t.order_time", startTime, endTime);

        }
        query.eq("t.del_flag", "N");
        query.orderByDesc("t.order_time");
        IPage<OrderBySellerDto> list = orderDao.queryPageListBySeller(page, query);
        if (list.getTotal() == 0) {
            return Result.of_error("没有订单！");
        }
        return Result.of_success(list);

    }

    /**
     * 用户中心-获取当前用户的成交单数
     *
     * @param
     * @return
     */
    @Override
    public Result<PurchaserUserCenterDto> userCenter(PurchaserUserCenterReq req) {
        PurchaserUserCenterDto dto = null;

        MallCustomerApiDto customer = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
        Map map = new HashMap();
        if (StringUtils.isNotBlank(req.getDay())) {
            map.put("day", req.getDay());
        }
        if (StringUtils.isNotBlank(req.getWeeks())) {
            map.put("weeks",req.getWeeks());
        }
        if (StringUtils.isNotBlank(req.getMonth())) {
            map.put("year", req.getMonth());
        }
        map.put("customerId", customer.getCustomerId());
        PurchaserUserCenterDto dtox = orderDao.getMoneyAndOrderTotalDto(map);
        if (dtox.getOrderTotal() == null) {
            return Result.of_error("查询失败！");
        }
        String orderTotal = dtox.getOrderTotal();
        BigDecimal transactionMoney = dtox.getTransactionMoney();
        BigDecimal divide = transactionMoney.divide(new BigDecimal(orderTotal), 2, BigDecimal.ROUND_DOWN);
        dto = departmentDao.selectByBasic(customer.getCustomerId());
        dto.setUseMoney(new BigDecimal(dto.getUseMoney()).setScale(2,BigDecimal.ROUND_DOWN).toString());
        dto.setOrderTotal(dtox.getOrderTotal());
        dto.setTransactionMoney(dtox.getTransactionMoney());
        dto.setMeanTransactionMoney(divide.toString());
        return Result.of_success(dto);

    }
    /**
     * 订单管理(条件分页查询)--买方
     * @param req
     * @return
     */
    @Override
    public Result<PageInfoResponse<OrderByPurchaserDto>> queryPageListByPurchaser(OrderRequest req) {
        // 调用gateway，获取调用数/错误率
        List<Map<String, String>> reqList = orderDao.getApiKey(getCustomer().getDepartmentId());
        JSONObject object = new JSONObject();
        object.put("reqList", reqList);
        String s = HttpsUtilsTest.submitPost(gatewayOrderLogNumError, object.toString());
        // 处理参数
        JSONObject params = JSONObject.parseObject(s);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.of_error("获取参数失败！");
        }
        Map<String, Object> param = new HashMap<>();
        if (StringUtils.isNotBlank(req.getDataName())) {
            param.put("dataName", req.getDataName());
        }
        if (StringUtils.isNotBlank(req.getPriceType())) {
            param.put("priceType", req.getPriceType());
        }
        if (StringUtils.isNotBlank(req.getGoodsStatus())) {
            param.put("goodsStatus", req.getGoodsStatus());
        }
        if (getCustomer().getCustomerId() != null && getCustomer().getDepartmentId() != null) {
            param.put("departmentId", getCustomer().getDepartmentId());
        }else {
            return Result.of_error("用户未认证，请认证！");
        }
        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        List<OrderByPurchaserDto> list = orderDao.queryPageListByPurchaser(param);
        // 封装参数
        List<Map> result = (List<Map>) JSONObject.parse(params.getString("data"));
        for (OrderByPurchaserDto orderByPurchaserDto : list) {
            for (Map map : result) {
                if (map.get("requestToken").equals(orderByPurchaserDto.getRequestToken())){
                    orderByPurchaserDto.setErrorRate(map.get("errorRate").toString());
                    orderByPurchaserDto.setCallNumber(map.get("callNumber").toString());
                }
            }
        }
        PageInfo<OrderByPurchaserDto> pageInfo = new PageInfo<>(list);
        return Result.of_success(pageInfo);


    }

    @Override
    public Result getApiKey() {
        MallCustomerApiDto customer = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
        List<Map<String, String>> apiKey = orderDao.getApiKey(customer.getCustomerId());
        if (apiKey.size() == 0) {
            return Result.of_error("apiKey为空！");
        }
        return Result.of_success(apiKey);
    }
    /**
     * 获取订单日志详情
     * @param req
     * @return
     */
    @Override
    public Result getLogDetails(LogInfoListReq req) {
        if (StringUtils.isEmpty(req.getApiKey())) {
            return Result.of_error("apiKey不能为空！");
        }
        if (StringUtils.isEmpty(req.getRequestToken())) {
            return Result.of_error("授权码不能为空！");
        }
        String post = HttpsUtilsTest.submitPost(gatewayOrderLogDetails, JSON.toJSONString(req));
        JSONObject params = JSONObject.parseObject(post);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))){
            return Result.of_error("获取日志详情失败！");
        }
        JSONObject object = JSONObject.parseObject(params.getString("data"));
        List<ApiReqLogDto> records = JSONObject.parseArray(object.getString("records"), ApiReqLogDto.class);

        return Result.ok(records);
    }

    /**
     * 重置盐值
     *
     * @param req
     * @return
     */
    @Override
    public Result updateSalt(SaltResetReq req) {
        //调用gatway
        JSONObject object = new JSONObject();
        object.put("authCode", req.getGoodsToken());
        object.put("createUser", getCustomer().getCustomerId());
        String s = HttpsUtilsTest.submitPost(gatewayUpSalt, object.toString());
        //处理返回的参数
        JSONObject params = JSONObject.parseObject(s);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.error("修改失败！");
        }
        String data = params.getString("data");
        JSONObject dataParam = JSONObject.parseObject(data);
        String salt = dataParam.getString("salt");
        // 修改参数
        Order order = new Order();
        order.setSaltValue(salt);
        order.setOrderId(req.getId());
        order.setUptTime(DateUtils.getToday());
        order.setUptPerson(getCustomer().getCustomerId() + "");
        if (orderDao.updateById(order) != 1) {
            return Result.of_error(ResultMsg.UPDATE_FAIL);
        }
        Map map = new HashMap();
        map.put("salt", salt);
        return Result.of_success(ResultMsg.SUCCESS, salt);
    }

    public MallCustomerApiDto getCustomer() {
        MallCustomerApiDto customer = (MallCustomerApiDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER);
        return customer;
    }
}