package com.jz.dm.mall.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.auth.AuthModeEnum;
import com.jz.common.enums.auth.AuthTypeEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.HttpsUtilsTest;
import com.jz.common.utils.NumberUtils;
import com.jz.common.utils.Result;
import com.jz.dm.mall.moduls.controller.pay.bean.dto.PayReq;
import com.jz.dm.mall.moduls.controller.pay.bean.req.AuthMallUserApiReq;
import com.jz.dm.mall.moduls.entity.*;
import com.jz.dm.mall.moduls.mapper.*;
import com.jz.dm.mall.moduls.service.PayService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.math.BigDecimal;
import java.util.*;

/**
 * @ClassName:
 * @Author: Carl
 * @Date: 2020/12/31
 * @Version:
 */
@Service("payService")
public class PayServiceImpl implements PayService {
    private static final Logger logger = LoggerFactory.getLogger(PayServiceImpl.class);

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private OrderPaymentDao orderPaymentDao;

    @Autowired
    private OrderGoodsInfoDao goodsInfoDao;

    @Autowired
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;

    @Autowired
    private FinanceTradeFlowDao financeTradeFlowDao;

    //@Autowired
    //private HttpsUtils httpsUtils;

    @Value("${gateway.gatewayGetAuth}")
    private String gatewayGetAuth;

    /**
     * 商品购买
     *
     * @param payReq
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result goodsPay(PayReq payReq) throws Exception{
        if (payReq.getDepartmentId() == null || payReq.getAssetsId() == null) {
            return Result.of_error("用户没关联企业，支付失败!");
        }
        // 新增订单关系表：订单表，订单支付表，订单商品表
        Result result = addOrder(payReq);
        Order order = (Order) result.getData();
        // 修改企业资产关系表：资产表，新增记录
        Result params = uptAssets(payReq, order);
        Map<String, String> map = (Map) params.getData();
        return Result.of_success(ResultMsg.SUCCESS,map);
    }



    /**
     * 制作有效时间
     * @param payReq
     * @return
     */
    public static Date getTime(PayReq payReq) {
        String purchaseMethod = payReq.getPurchaseMethod();
        Date now = DateUtils.getToday();
        if ("01".equals(purchaseMethod)) {
            Date nowAddYear = DateUtils.getAfterYear(now);
            return nowAddYear;
        }else if ("02".equals(purchaseMethod)){
            Date nowAddSeason = DateUtils.getAfterSeason(now);
            return nowAddSeason;
        }else if("03".equals(purchaseMethod)) {
            Date nowAddMonth = DateUtils.getAfterMonth(now);
            return nowAddMonth;
        }else {
            return null;
        }
    }

    /**
     * 授权: 获取盐值和授权码
     */
    public Result getAuthorization(PayReq payReq, Order order) throws Exception{
        // 年季月次 商品
        AuthMallUserApiReq apiReq = new AuthMallUserApiReq();
        if ("04".equals(payReq.getPurchaseMethod())) {
            apiReq.setAuthMode(AuthModeEnum.POWER_CALL_MODE);
            apiReq.setValidStartTime(null);
            apiReq.setValidEndTime(null);
        }else {
            apiReq.setAuthMode(AuthModeEnum.RECORD_TIME_MODE);
            apiReq.setValidStartTime(order.getTakeEffectTime());
            apiReq.setValidEndTime(order.getInvalidTime());
        }
        apiReq.setApiKey(payReq.getApiKey());
        apiReq.setAuthType(AuthTypeEnum.DATA_BANK_AUTH);
        apiReq.setOrgCode("WAISJYX40E3e"); // TODO
        apiReq.setOrgType("OUT"); // TODO
        apiReq.setUserId(payReq.getCustomerId() + "");

        String post = HttpsUtilsTest.submitPost(gatewayGetAuth, JSON.toJSONString(apiReq));
        JSONObject params = JSONObject.parseObject(post);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))){
            throw ResponseException.of_error("授权失败！");
        }
        logger.info("###########################购买商品：授权成功！##############################");
        // 获取到盐值和token
        String data = params.getString("data");
        JSONObject parse = JSONObject.parseObject(data);
        String authCode = parse.getString("authCode");
        String salt = parse.getString("salt");
        if (StringUtils.isEmpty(authCode) || StringUtils.isEmpty(salt)) {
            return Result.of_error("授权失败，未获取到授权码和盐值！");
        }
        Map<String, String> map = new HashMap();
        map.put("authCode",authCode);
        map.put("salt",salt);
        return Result.ok(map);
    }

    /**
     * 新增订单关系表：订单表，订单支付表，订单商品表
     */
    public Result addOrder(PayReq payReq) throws Exception{
        // 新增订单主表
        Order order = new Order();
        order.setOrderNumber(NumberUtils.getOrderNumber(payReq.getCustomerId() + ""));
        order.setCustomerId(payReq.getCustomerId());
        order.setOrderMoney(payReq.getDataPrice());
        order.setOrderStatus("02");
        order.setPaymentMoney(payReq.getDataPrice());
        order.setOrderTime(DateUtils.getToday());
        order.setPaymentTime(order.getOrderTime());
        order.setPaymentMethod("01");
//        order.setSellerId();  // TODO  卖家id
//        order.setDistrictMoney(); //TODO  优惠金额
        order.setTakeEffectTime(order.getOrderTime());
        order.setApiKey(payReq.getApiKey());
        //处理生效期
        Date time = getTime(payReq);
        if (time != null) {
            order.setInvalidTime(time);
        }
        order.setPurchaseMethod(payReq.getPurchaseMethod());
        order.setPriceType(payReq.getPriceType());
        order.setCrePerson(payReq.getCustomerId() + "");
        order.setCreTime(DateUtils.getToday());
        if (orderDao.insert(order) != 1) {
            throw ResponseException.of_error("新增订单失败！");
        }
        logger.info("###########################购买商品：新增订单成功！##############################");
        // 新增订单支付表/订单商品表
        if (order.getOrderId() != null) {
            OrderPayment orderPayment = new OrderPayment();
//            orderPayment.setSellerId(); // TODO
            orderPayment.setPaymentMoney(payReq.getDataPrice());
            orderPayment.setPaymentTime(DateUtils.getToday());
            orderPayment.setPaymentMethod("01");
//            orderPayment.setDistrictMoney(); //TODO
            orderPayment.setPaymentStatus(order.getOrderStatus());
            orderPayment.setPurchaseMethod(payReq.getPurchaseMethod());
            orderPayment.setCreTime(DateUtils.getToday());
            orderPayment.setCrePerson(payReq.getCustomerId()+"");
            if (orderPaymentDao.insert(orderPayment) != 1) {
                throw ResponseException.of_error("新增订单支付表失败！");
            }
            logger.info("###########################购买商品：新增订单支付表成功！##############################");
            // 订单商品表
            OrderGoodsInfo goodsInfo = new OrderGoodsInfo();
            goodsInfo.setOrderId(order.getOrderId());
            goodsInfo.setDataGoodsId(payReq.getDataGoodsId());
            goodsInfo.setCreTime(DateUtils.getToday());
            goodsInfo.setCrePerson(payReq.getCustomerId()+"");
            // 新增
            if (goodsInfoDao.insert(goodsInfo) != 1) {
                throw ResponseException.of_error("新增订单商品表失败！");
            }
            logger.info("###########################购买商品：新增订单商品表成功！##############################");
        }
        return Result.ok(order);
    }

    /**
     * 修改企业资产关系表：资产表，新增记录
     */
    public Result uptAssets(PayReq payReq, Order order) throws Exception{
        // 获取企业资产信息
        FinanceCustomerAssets assets = financeCustomerAssetsDao.findById(payReq.getAssetsId());
//        Long q = financeCustomerAssetsDao.findByOne(l);
        // 可用金额减去支付金额
        BigDecimal bigDecimal = new BigDecimal(assets.getUseMoney()+ "");
        BigDecimal bigDecima2 = new BigDecimal(payReq.getDataPrice()+ "");
        if (bigDecimal.compareTo(bigDecima2) == -1){
            return Result.error("金额不足！请充值！");
        }
        BigDecimal subtract = bigDecimal.subtract(bigDecima2);

        assets.setUseMoney(subtract);
        assets.setTotalMoney(subtract);
        assets.setUptTime(DateUtils.getToday());
        assets.setUptPerson(payReq.getCustomerId()+ "");
        if (financeCustomerAssetsDao.updateById(assets) != 1) {
            throw ResponseException.of_error("修改资产金额失败！");
        }
        logger.info("###########################购买商品：修改资产金额成功！##############################");
        // 授权：获取到盐值和token，更新订单表
        Result authorization = getAuthorization(payReq, order);
        Map<String, String> map= (Map) authorization.getData();
        order.setGoodsToken(map.get("authCode"));
        order.setSaltValue(map.get("salt"));
        order.setUptTime(DateUtils.getToday());
        order.setUptPerson(payReq.getCustomerId() + "");
        if (orderDao.updateById(order) != 1) {
            throw ResponseException.of_error("修改订单盐值失败！");
        }
        logger.info("###########################购买商品：修改订单盐值成功！##############################");
        // 新增记录表
        Long assetsId = assets.getAssetsId();
        if (assetsId!= null) {
            FinanceTradeFlow flow = new FinanceTradeFlow();
            flow.setAssetsId(assetsId);
            flow.setOrderId(order.getOrderId());
            flow.setTradeFlowNumber(NumberUtils.getDealNum());// 交易流水编号
            flow.setTradeMoney(payReq.getDataPrice());
            flow.setTradeType("03");
            flow.setCreTime(DateUtils.getToday());
            flow.setCrePerson(payReq.getCustomerId() + "");
            flow.setTradeFlowNumber(NumberUtils.getDealNum());
            if (financeTradeFlowDao.insert(flow) != 1) {
                throw ResponseException.of_error("新增记录表失败！");
            }
            logger.info("###########################购买商品：新增记录表成功！##############################");
        }
        return Result.ok(map);
    }
}
