package com.jz.dm.mall.moduls.service.impl;

import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.utils.Result;
import com.jz.common.utils.SMSUtils;
import com.jz.common.utils.ValidateCodeUtils;
import com.jz.dm.mall.moduls.service.ValidateCodeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

/**
 * @ClassName:
 * @Author: Carl
 * @Date: 2021/1/28
 * @Version:
 */
@Service("validateCodeService")
public class ValidateCodeServiceImpl implements ValidateCodeService {
    @Autowired
    private RedisTemplate redisTemplate;

    @Value("${SMS.msgSign}")
    private String msgSign;

    @Value("${SMS.template}")
    private String template;

    @Value("${SMS.accessKeyId}")
    private String accessKeyId;

    @Value("${SMS.accessKeySecret}")
    private String accessKeySecret;

    /**
     * 发送验证码
     *
     * @param telephone
     * @return
     */
    @Override
    public Result send4Code(String telephone) {
        String key = RedisMessageConstant.SENDTYPE_GETPWD + "_" + telephone;
        // 通过手机号从redis获取验证码
        String codeInRedis = (String) redisTemplate.opsForValue().get(key);

        if (codeInRedis!=null && !codeInRedis.equals("")) {
            // redis中的验证码还未过期
            return Result.of_error("验证码发送失败！");
        } else {
            // redis中无此手机号的验证码，发送验证码
            Integer code = ValidateCodeUtils.generateValidateCode(6);
            SMSUtils.sendMessage(template,code,telephone,accessKeyId,accessKeySecret,msgSign);
            redisTemplate.opsForValue().set(key,code,5, TimeUnit.MINUTES);
            return Result.of_success("验证码发送成功！");
        }
    }

    /**
     * 修改密码发送的验证码
     * @param telephone
     * @return
     */
    @Override
    public Result sendForLogin(String telephone) {
        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
        // 通过手机号从redis获取验证码
        String codeInRedis = (String) redisTemplate.opsForValue().get(key);

        if (codeInRedis!=null && !codeInRedis.equals("")) {
            // redis中的验证码还未过期
            return Result.of_error("验证码发送失败！");
        } else {
            // redis中无此手机号的验证码，发送验证码
            Integer code = ValidateCodeUtils.generateValidateCode(6);
            SMSUtils.sendMessage(template,code,telephone,accessKeyId,accessKeySecret,msgSign);
            redisTemplate.opsForValue().set(key,code,5, TimeUnit.MINUTES);
            return Result.ok("验证码发送成功！");
        }
    }
}
