package com.jz.manage.moduls.controller.goods;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.BaseController;
import com.jz.manage.moduls.controller.goods.bean.request.AddCateGoryReq;
import com.jz.manage.moduls.controller.goods.bean.request.CategoryAndSceneReq;
import com.jz.manage.moduls.controller.goods.bean.request.CategoryDetailReq;
import com.jz.manage.moduls.controller.goods.bean.request.DataGoodsByCategoryReq;
import com.jz.manage.moduls.entity.DataGoodsCategory;
import com.jz.manage.moduls.service.DataGoodsCategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 数据商品分类(行业)(TDataGoodsCategory)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:32
 */
@RestController
@RequestMapping("dataGoodsCategory")
@Api(tags = "平台--商品管理--分类管理--controller")
public class DataGoodsCategoryController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private DataGoodsCategoryService categoryService;

    @GetMapping(value = "/findByCategory")
    @ApiOperation(value = "查询数据商品分类", notes = "查询所有的商品分类")
    public Result<List<DataGoodsCategory>> findByCategory() throws Exception{
        return categoryService.findByCategory();
    }

    /**
     * 查询场景分类
     * @return
     */
    @GetMapping(value = "/findByScene")
    @ApiOperation(value = "查询场景分类", notes = "查询所有的场景分类")
    public Result findByScene() {
        return categoryService.findByScene();
    }

    /**
     * 添加分类
     * @param category
     * @return
     */
    @PostMapping(value = "/addCategory")
    @ApiOperation(value = "商品分类管理--添加分类", notes = "添加一级分类/子级分类")
    public Result addCategory(@RequestBody @Valid AddCateGoryReq category) {
        try {
            return categoryService.addCategory(category);
        }catch (Exception e) {
            e.printStackTrace();
            return Result.of_error(ResultMsg.FAILURE);
        }
    }

    /**
     * 编辑分类
     * @param category
     * @return
     */
    @PostMapping(value = "/editCateGoryByParent")
    @ApiOperation(value = "商品分类管理--编辑分类", notes = "编辑一级分类/子级分类")
    public Result editCategory(@RequestBody @Valid AddCateGoryReq category) {
        try {
            return categoryService.editCategory(category);
        }catch (Exception e) {
            e.printStackTrace();
            return Result.of_error(ResultMsg.FAILURE);
        }
    }

    /**
     * 获取分类详情
     * @param req
     * @return
     */
    @PostMapping("/getCategoryDetail")
    @ApiOperation(value = "商品分类管理--获取分类详情")
    public Result getCategoryDetail(@RequestBody @Valid CategoryDetailReq req) {
        return categoryService.getCategoryDetail(req);
    }

    /**
     * 条件查询商品分类
     * @param search
     * @return
     */
    @PostMapping(value = "/selectByCategory")
    @ApiOperation(value = "商品分类管理--条件查询商品分类")
    public Result selectByCategory(@RequestParam(name = "search",required = false) String search) {
        return categoryService.selectByCategory(search);
    }

    @PostMapping(value = "/deleteByCategory")
    @ApiOperation(value = "商品分类管理--删除分类")
    public Result deleteByCategory(@RequestBody @Valid CategoryAndSceneReq req) {
        try {
            return categoryService.deleteByCategory(req);
        }catch (Exception e) {
            e.printStackTrace();
            return Result.of_error(ResultMsg.DELETE_FAIL);
        }
    }
}