package com.jz.manage.moduls.controller.goods;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.constant.Constants;

import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.BaseController;
import com.jz.manage.moduls.controller.goods.bean.dto.*;
import com.jz.manage.moduls.controller.goods.bean.request.*;
import com.jz.manage.moduls.entity.DataGoods;
import com.jz.manage.moduls.service.DataGoodsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

/**
 * 数据商品(dataGoods)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:30
 */
@RestController
@RequestMapping("dataGoods")
@Api(tags = "平台--商品管理--商品列表--controller")
public class DataGoodsController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private DataGoodsService dataGoodsService;



    /**列表查询数据商品
	 * @param dataGoodsListRequest
	 * @return
	 */
	@PostMapping(value = "/findList")
	@ApiOperation(value = "商品列表--列表查询数据")
	public Result<IPage<DataGoodsListDto>> findList(@RequestBody DataGoodsListRequest dataGoodsListRequest, HttpServletRequest httpRequest){
		try {
			return dataGoodsService.findList(dataGoodsListRequest, httpRequest);
		} catch (Exception e) {
			e.printStackTrace();
			return Result.of_error("查询失败！");
		}

	}
	/**列表查询审核数据商品
	 * @param req
	 * @return
	 */
	@PostMapping(value = "/findByAudit")
	@ApiOperation(value = "商品审核管理--列表查询数据")
	public Result<IPage<DataGoodsAuditDto>> findByAudit(@RequestBody DataGoodsAuditListReq req, HttpServletRequest httpRequest){
        return dataGoodsService.findByAudit(req,httpRequest);
	}

	@GetMapping(value = "/findByDataTypeList")
	@ApiOperation(value = "商品列表--获取数据类型", notes = "调用接口")
	public Result findByDataTypeList() {
		return dataGoodsService.findByDataTypeList();
	}

	/**
	 * 商品审核管理--商品审核
	 * @param req
	 * @return
	 */
	@PostMapping(value = "/auditDataGoods")
	@ApiOperation(value = "商品审核管理--商品审核")
	public Result auditDataGoods(@RequestBody @Valid DataGoodsAuditReq req) {
        return dataGoodsService.auditDataGoods(req);
	}



	
	/**新增数据商品
	 * @param
	 * @return
	 */
//	@PostMapping("/uploadApi")
//	@ApiOperation(value = "自营数据api上传", notes = "api上传")
//	public Result uploadApi(@RequestBody DataApiUpReq dataApiUpReq) throws Exception{
//		Result result = new Result();
//		try{
//			result = dataGoodsService.uploadApi(dataApiUpReq);
//		}catch (Exception e) {
//			e.printStackTrace();
//			result = new Result(false, "api上传失败");
//		}
//		return result;
//	}

//	@PostMapping("/getJoinType")
//	@ApiOperation(value = "调用gateway获取joinType")
//	public Result getJoinType(@RequestParam(name = "parentKey") String parentKey) {
//		Result result = dataGoodsService.getJoinType(parentKey);
//		return null;
//	}


	/**
	 * 商品上下架

	 * @return
	 */
	@PostMapping("/editStatus")
	@ApiOperation(value = "商品列表--商品上下架")
	public Result editStatus(@RequestBody @Valid DataGoodsEditReq req) {
		try {
			return dataGoodsService.editStatus(req);
		}catch (Exception e) {
			e.printStackTrace();
			return Result.of_error(ResultMsg.UPDATE_FAIL);
		}
	}

	/**
	 * 商品列表--浏览授权
	 * @param roleReq
	 * @return
	 */
	@PostMapping("/browseByAuthorization")
	@ApiOperation(value = "商品列表--浏览授权")
	public Result browseByAuthorization(@RequestBody @Valid DataGoodsByRoleReq roleReq) {
		return dataGoodsService.browseByAuthorization(roleReq);
	}

	/**
	 * 商品列表--浏览授权--查询授权
	 * @param dataGoodsId
	 * @return
	 */
	@GetMapping("/findByAuthorization")
	@ApiOperation(value = "商品列表--浏览授权--查询授权")
	public Result findByAuthorization(@RequestParam(name = "dataGoodsId") Long dataGoodsId) {
		return dataGoodsService.findByAuthorization(dataGoodsId);
	}

	/**
	 *
	 * @param
	 * @return
	 */
	@PostMapping("/useByAuthorization")
	@ApiOperation(value = "商品列表--使用授权")
	public Result useByAuthorization(@RequestBody GoodsByAuthorizationReq req) {
		try {
			return dataGoodsService.useByAuthorization(req);
		}catch (Exception e) {
			e.printStackTrace();
			return Result.of_error(ResultMsg.FAILURE);
		}
	}

	@PostMapping("/findByDepartment")
	@ApiOperation(value = "商品列表--使用授权/续费--搜索企业")
	public Result findByDepartment(@RequestBody DepartmentGoodsReq req) {
		return dataGoodsService.findByDepartment(req);
	}



	/**
	 * 根据主键查询商品详情-api
	 * @param id
	 * @return
	 */
	@GetMapping("/getDataDetails/{id}")
	@ApiOperation(value = "商品列表--查看商品详情")
	public Result findById(@PathVariable(value = "id") Long id) throws Exception{
		if (id != null) {
			return dataGoodsService.selectById(id);
		}
		return Result.of_error("查询失败！");
	}


	/**
	 * 获取api列表
	 * @param req
	 * @return
	 */
	@PostMapping("/getApiGoods")
	@ApiOperation(value = "制作商品-获取api列表",notes = "dmp制作api，发送到数据银行")
	public Result getAPIList(@RequestBody @Valid SendDataReq req) {
		return dataGoodsService.getAPIList(req);
	}

	/**
	 * 获取制作的商品
	 * @param req
	 * @return
	 */
	@PostMapping("/getDataGoodsNewsList")
	@ApiOperation(value = "制作商品-获取制作的商品")
	public Result<IPage<ApiDataGoodsListDto>> getDataGoodsNewsList(@RequestBody @Valid DataGoodsByMakeReq req) {
		return dataGoodsService.getDataGoodsNewsList(req);
	}


	@PostMapping("/updateSendStatus")
	@ApiOperation(value = "制作商品-退回数据")
	public Result updateSendStatus(@RequestBody @Valid ApiInterfaceDetailReq req) {
		return dataGoodsService.updateSendStatus(req);
	}

	/**
	 * 获取API详情
	 * @param req
	 * @return
	 */
	@ApiOperation(value = "制作商品-API详情查询",notes = "调用接口")
	@PostMapping(value = "/getApiInterfaceDetail")
	public Result<DataDetailsDto> getApiInterfaceDetail(@RequestBody @Valid ApiInterfaceDetailReq req) {
		return dataGoodsService.getApiInterfaceDetail(req);
	}
	/**
	 * 制作商品
	 * @param req
	 * @return
	 */
	@PostMapping("/makeByGoods")
	@ApiOperation(value = "制作商品--编辑/制作商品",notes = "平台方制作商品")
	public Result makeByGoods(@RequestBody @Valid DataApiUpReq req) {
		try {
			return dataGoodsService.makeByGoods(req);
		} catch (Exception e) {
			e.printStackTrace();
			return Result.of_error("制作失败！");
		}
	}
}