package com.jz.manage.moduls.controller.order;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.BaseController;
import com.jz.manage.moduls.controller.order.bean.dto.OrderDetailDto;
import com.jz.manage.moduls.controller.order.bean.dto.OrderDto;
import com.jz.manage.moduls.controller.order.bean.req.ApiInterfaceDetailReq;
import com.jz.manage.moduls.controller.order.bean.req.OrderRequest;
import com.jz.manage.moduls.controller.order.bean.req.SaltResetReq;
import com.jz.manage.moduls.service.OrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

/**
 * 订单表(TOrder)表控制层
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:41
 */
@RestController
@RequestMapping("/order")
@Api(tags = "平台--订单管理--controller")
public class OrderController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private OrderService orderService;

    /**
     * 订单管理列表（分页查询）
     *
     * @author Bellamy
     */
    @PostMapping(value = "/list")
    @ApiOperation(value = "订单管理列表（分页查询）", notes = "订单列表（分页查询）")
    public Result<IPage<OrderDto>> queryPageList(@RequestBody OrderRequest orderRequest, HttpServletRequest req) throws Exception {
            return orderService.queryListOrderInfo(orderRequest, req);
    }

    /**
     * 根据订单主键查询，订单详情
     *
     * @param orderId
     * @author Bellamy
     */
    @GetMapping(value = "/getOrderDetail")
    @ApiOperation(value = "API订单--订单详情", notes = "订单详情")
    public Result<OrderDetailDto> getOrderDetail(@RequestParam(value = "orderId") String orderId, HttpServletRequest req) throws Exception {
        Result<OrderDetailDto> result = new Result<OrderDetailDto>();
        if (StringUtils.isNotEmpty(orderId)) {
            return orderService.queryOrderDetail(orderId);
        } else {
            result.setMessage("参数为不能为空！");
        }
        return result;
    }

    @PostMapping(value = "/getOrderApiDetail")
    @ApiOperation(value = "API订单--订单详情--接口文档")
    public Result getOrderApiDetail(@RequestBody @Valid ApiInterfaceDetailReq req) {
        return orderService.getOrderApiDetail(req);
    }

    /**
     * 订单详情--重置盐值
     * @param req
     * @return
     */
    @PostMapping(value = "/resetSalt")
    @ApiOperation(value = "订单详情---重置盐值", notes = "重置盐值")
    public Result resetSalt(@RequestBody @Valid SaltResetReq req) {
        Result result = new Result();
        if (StringUtils.isNotEmpty(req.getGoodsToken()) && req.getId() != null) {
            result = orderService.updateSalt(req);
        }else {
            result.setMessage("参数不能为空！");
        }
        return result;
    }
}