package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.config.bean.req.BannerAddAndEditReq;
import com.jz.manage.moduls.controller.config.bean.req.BannerConfigListReq;
import com.jz.manage.moduls.entity.Banner;
import com.jz.manage.moduls.mapper.BannerDao;
import com.jz.manage.moduls.service.BannerServer;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;


/**
 * @ClassName:
 * @Author: Carl
 * @Date: 2021/2/7
 * @Version:
 */
@Service("bannerServer")
public class BannerServerImpl implements BannerServer {

    @Autowired
    private BannerDao bannerDao;

    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 查询全部广告
     *
     * @return
     */
    @Override
    public Result<IPage<Banner>> findList(BannerConfigListReq req) {
        IPage<Banner> page = new Page<>(req.getPageNum(), req.getPageSize());

        QueryWrapper<Banner> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(req.getSearch())) {
            queryWrapper.and(s -> s.like("t.bannerName", req.getSearch()));
        }
        if (StringUtils.isNotEmpty(req.getBannerLocation())) {
            queryWrapper.eq("t.bannerLocation", req.getBannerLocation());
        }
        if (StringUtils.isNotEmpty(req.getBannerStatus())) {
            queryWrapper.eq("t.bannerStatus", req.getBannerStatus());
        }
        if (StringUtils.isNotBlank(req.getBannerStart()) && StringUtils.isNotBlank(req.getBannerEnd())) {
            String startTime = req.getBannerStart() + " 00:00:00";
            String endTime = req.getBannerEnd() + " 23:59:59";
            queryWrapper.ge("t.bannerStart",startTime);
            queryWrapper.le("t.bannerEnd",endTime);
        }
        queryWrapper.eq("t.delFlag", DelFlagEnum.NO.getValue());
        queryWrapper.orderByDesc("t.uptTime");
        IPage<Banner> list = bannerDao.selectByList(page, queryWrapper);
        if (list.getTotal() != 0) {
            return Result.of_success(list);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 查询广告
     *
     * @param bannerId
     * @return
     */
    @Override
    public Result findByBanner(Long bannerId) {
        if (bannerId != null) {
            QueryWrapper<Banner> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("banner_id", bannerId);
            queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
            Banner banner = bannerDao.selectOne(queryWrapper);
            if (banner != null) {
                return Result.ok(banner);
            }
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 新增广告
     *
     * @param req
     * @return
     */
    @Override
    public Result addBanner(BannerAddAndEditReq req) {
        Banner banner = new Banner();
        banner.setCreTime(DateUtils.getToday());
        banner.setCrePerson(userDto().getUserId() + "");
        banner.setDelFlag(DelFlagEnum.NO.getValue());
        BeanUtils.copyProperties(req,banner);
        if (bannerDao.insert(banner) != 1) {
            return Result.of_error(ResultMsg.INSERT_FAIL);
        }
        return Result.of_success(ResultMsg.INSERT_SUCCESS);
    }

    /**
     * 编辑广告
     *
     * @param req
     * @return
     */
    @Override
    public Result editBanner(BannerAddAndEditReq req) {
        Banner banner = new Banner();
        banner.setUptTime(DateUtils.getToday());
        banner.setUptPerson(userDto().getUserId() + "");
        BeanUtils.copyProperties(req,banner);
        if (bannerDao.updateById(banner) != 1) {
            return Result.of_error(ResultMsg.UPDATE_FAIL);
        }
        return Result.of_success(ResultMsg.UPDATE_SUCCESS);
    }

    /**
     * 删除广告
     *
     * @param bannerId
     * @return
     */
    @Override
    public Result deleteBanner(Long bannerId) {
        QueryWrapper<Banner> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("banner_id", bannerId);
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        Banner banner = bannerDao.selectOne(queryWrapper);
        if (banner != null) {
            banner.setDelFlag(DelFlagEnum.YES.getValue());
            if (bannerDao.updateById(banner) != 1) {
                return Result.of_error(ResultMsg.DELETE_FAIL);
            }
            return Result.of_error(ResultMsg.DELETE_SUCCESS);
        }
        return Result.of_error(ResultMsg.DELETE_FAIL);
    }


    public SysUserDto userDto() {
        SysUserDto userDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return userDto;
    }
}
