package com.jz.manage.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.enums.PaymentMethodEnum;
import com.jz.common.enums.TradeFlowEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.*;
import com.jz.manage.moduls.controller.goods.bean.dto.DataBasicDto;
import com.jz.manage.moduls.controller.goods.bean.dto.DataGoodsApiDto;
import com.jz.manage.moduls.controller.goods.bean.dto.DataInterfaceDto;
import com.jz.manage.moduls.controller.goods.bean.dto.DepartmentDataGoodsDto;
import com.jz.manage.moduls.controller.goods.bean.request.AuthRenewUpdateReq;
import com.jz.manage.moduls.controller.goods.bean.request.DepartmentByRenewReq;
import com.jz.manage.moduls.controller.goods.bean.request.DepartmentDataGoodsReq;
import com.jz.manage.moduls.controller.goods.bean.request.GoodsByAuthorizationReq;
import com.jz.manage.moduls.entity.FinanceCustomerAssets;
import com.jz.manage.moduls.entity.FinanceTradeFlow;
import com.jz.manage.moduls.entity.Order;
import com.jz.manage.moduls.entity.OrderPayment;
import com.jz.manage.moduls.mapper.*;
import com.jz.manage.moduls.service.DataGoodsApiService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * api商品(TDataGoodsApi)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:31
 */
@Service("dataGoodsApiService")
public class DataGoodsApiServiceImpl implements DataGoodsApiService {
    private static final Logger logger = LoggerFactory.getLogger(DataGoodsApiServiceImpl.class);
    @Autowired
    private DataGoodsApiDao tDataGoodsApiDao;

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private OrderPaymentDao orderPaymentDao;

    @Autowired
    private FinanceTradeFlowDao financeTradeFlowDao;

    @Autowired
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;

    @Autowired
    private RedisTemplate redisTemplate;

    @Value("${gateway.updateAuthRenew}")
    private String updateAuthRenew;

    /**
     * 主键查询数据详情
     *
     * @param id
     * @return
     */
    @Override
    public Result selectById(Long id) {
        List<DataGoodsApiDto> dataGoodsApiDto = tDataGoodsApiDao.findById(id);
        if (dataGoodsApiDto.size() == 0) {
            return Result.error(StatusCode.ERROR, "查询异常！");
        }
        // 参数信息
        List<DataGoodsApiDto> list1 = new ArrayList<>();
        List<DataGoodsApiDto> list2 = new ArrayList<>();
        List<DataGoodsApiDto> list3 = new ArrayList<>();
        List<DataGoodsApiDto> list4 = new ArrayList<>();
        List<DataGoodsApiDto> list5 = new ArrayList<>();
        Map<String, Object> map = new HashMap<>();
        // 基本信息
        DataGoodsApiDto dataGoods = dataGoodsApiDto.get(0);
        DataBasicDto dataBasicDto = new DataBasicDto();
        BeanUtils.copyProperties(dataGoods, dataBasicDto);// 赋值

        // 接口文档
        DataInterfaceDto dataInterfaceDto = new DataInterfaceDto();
        BeanUtils.copyProperties(dataGoods, dataInterfaceDto);


        for (DataGoodsApiDto goodsApiDto : dataGoodsApiDto) {
            // 固定参数
            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("01")) {
                list1.add(goodsApiDto);
            }
            // 请求参数
            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("02")) {
                list2.add(goodsApiDto);
            }
            // 响应参数
            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("03")) {
                list3.add(goodsApiDto);
            }
            // 请求头参数
            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("04")) {
                list4.add(goodsApiDto);
            }
            // 状态码参数
            if (goodsApiDto.getParamsDiffNum()!= null && goodsApiDto.getParamsDiffNum().equals("05")) {
                list5.add(goodsApiDto);
            }
            // api返回数据样例
            if (!StringUtils.isEmpty(goodsApiDto.getReturnDataExample())) {
                map.put("returnDataExample", goodsApiDto.getReturnDataExample()); // 响应示例
            }
        }
        map.put("dataParam", dataBasicDto); // 基本信息
        map.put("dataInterfaceParam", dataInterfaceDto); // 接口文档
        map.put("pubParam", list1); // 公共参数
        map.put("reqParam", list2); // 请求参数
        map.put("resParam", list3); // 响应参数
        map.put("reqHeadParam", list4); // 请求头参数
        map.put("statusCodeParam", list5); // 状态码参数
        Result result = new Result();
        result.setData(map);
        return Result.of_success(ResultMsg.SUCCESS, result);
    }

    /**
     * 条件查询企业商品列表
     *
     * @param req
     * @return
     */
    @Override
    public Result<IPage<DepartmentDataGoodsDto>> findByDepartmentGoods(DepartmentDataGoodsReq req) {
        IPage<DepartmentDataGoodsDto> page = new Page<>(req.getPageNum(),req.getPageSize());

        QueryWrapper<DepartmentDataGoodsDto> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(req.getSearch())) {
            queryWrapper.and(s -> s.like("t3.department_name",req.getSearch())
                            .or().like("t5.data_name", req.getSearch())
                    );
        }
        if (req.getCategoryId() != null) {
            queryWrapper.eq("t5.category_id",req.getCategoryId());
        }
        queryWrapper.eq("t1.del_flag", DelFlagEnum.NO.getValue());
        queryWrapper.orderByDesc("t1.take_effect_time");
        IPage<DepartmentDataGoodsDto> list = tDataGoodsApiDao.selectByList(page, queryWrapper);
        if (list.getTotal() == 0) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        return Result.of_success(list);
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result DepartmentByRenew(DepartmentByRenewReq req) throws Exception {

        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("order_id", req.getOrderId());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        // order更新时间
        Order order = orderDao.selectOne(queryWrapper);
        if (order == null) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        // gateway 更新时间
        upGateWayTime(req,order.getGoodsToken());
        // order更新时间
        addOrderPayment(req,order);
        // 修改资产关系表
        uptAssets(req,req.getOrderId());
        return Result.ok();
    }

    public Result upGateWayTime(DepartmentByRenewReq req,String authCode) {
        AuthRenewUpdateReq renewUpdateReq = new AuthRenewUpdateReq();
        renewUpdateReq.setAuthCode(authCode);
        renewUpdateReq.setValidEndTime(req.getInvalidTime());
        String post = HttpsUtilsTest.submitPost(updateAuthRenew, JSON.toJSONString(renewUpdateReq));
        JSONObject params = JSONObject.parseObject(post);
        if (!params.get("code").equals(200)){
            throw ResponseException.of_error("授权失败！");
        }
        logger.info("###########################购买商品：授权成功！##############################");
        return Result.ok();
    }

    /**
     * 修改order更新时间/新增支付表时间
     * @param req
     * @return
     */
    public Result addOrderPayment(DepartmentByRenewReq req, Order order) {
        order.setInvalidTime(req.getInvalidTime());
        order.setPurchaseMethod(req.getPurchaseMethod());
        order.setOrderMoney(req.getDataPrice());
        order.setPaymentMoney(req.getDataPrice());
        order.setUptPerson(getUserId() + "");
        order.setUptTime(DateUtils.getToday());
        if (orderDao.updateById(order) != 1) {
            throw ResponseException.of_error("修改订单失败！");
        }
        logger.info("#############################续费：修改订单失败！##################################");
        //新增订单支付
        OrderPayment orderPayment = new OrderPayment();
        orderPayment.setOrderId(order.getOrderId());
        orderPayment.setSellerId(req.getCustomerId());
        orderPayment.setPaymentMoney(req.getDataPrice());
        orderPayment.setPaymentTime(DateUtils.getToday());
        orderPayment.setPaymentMethod(PaymentMethodEnum.YE.getValue());
        orderPayment.setPaymentTime(DateUtils.getToday());
        orderPayment.setPaymentStatus(order.getOrderStatus());
        orderPayment.setPurchaseMethod(req.getPurchaseMethod());
        orderPayment.setCrePerson(getUserId() +"");
        orderPayment.setCreTime(DateUtils.getToday());
        orderPayment.setDelFlag(DelFlagEnum.NO.getValue());
        if (orderPaymentDao.insert(orderPayment) != 1) {
            throw ResponseException.of_error("新增订单支付表失败！");
        }
        logger.info("#############################续费：新增订单支付表失败！##################################");
        return Result.ok();
    }

    /**
     * 修改资产关系表
     * @param req
     * @param orderId
     * @return
     * @throws Exception
     */
    public Result uptAssets(DepartmentByRenewReq req, Long orderId) throws Exception{
        // 获取企业资产信息
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("assets_id", req.getAssetsId());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        FinanceCustomerAssets assets = financeCustomerAssetsDao.selectOne(queryWrapper);
        // 可用金额减去支付金额
        BigDecimal bigDecimal = new BigDecimal(assets.getUseMoney()+ "");
        BigDecimal bigDecima2 = new BigDecimal(req.getDataPrice()+ "");
        if (bigDecimal.compareTo(bigDecima2) == -1){
            return Result.error("金额不足！请充值！");
        }
        BigDecimal subtract = bigDecimal.subtract(bigDecima2);
        // 修改金额
        assets.setUseMoney(subtract);
        assets.setTotalMoney(subtract);
        assets.setUptTime(DateUtils.getToday());
        assets.setUptPerson(req.getCustomerId()+ "");
        if (financeCustomerAssetsDao.updateById(assets) != 1) {
            throw ResponseException.of_error("修改资产金额失败！");
        }
        logger.info("###########################购买商品：修改资产金额成功！##############################");
        // 新增记录表
        Long assetsId = assets.getAssetsId();
        if (assetsId!= null) {
            FinanceTradeFlow flow = new FinanceTradeFlow();
            flow.setAssetsId(assetsId);
            flow.setOrderId(orderId);
            flow.setTradeFlowNumber(NumberUtils.getDealNum());// 交易流水编号
            flow.setTradeMoney(req.getDataPrice());
            flow.setTradeType(TradeFlowEnum.XF.getValue());
            flow.setCreTime(DateUtils.getToday());
            flow.setCrePerson(req.getCustomerId() + "");
            flow.setTradeFlowNumber(NumberUtils.getDealNum());
            flow.setDelFlag(DelFlagEnum.NO.getValue());
            if (financeTradeFlowDao.insert(flow) != 1) {
                throw ResponseException.of_error("新增记录表失败！");
            }
            logger.info("###########################购买商品：新增记录表成功！##############################");
        }
        return Result.ok();
    }

    /**
     * 获取当前用户
     * @return
     */
    public Long getUserId() {
        SysUserDto sysUserDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return sysUserDto.getUserId();
    }

}