package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.DataGoodsByRoleEnum;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.enums.YNEnum;
import com.jz.common.enums.auth.CategoryByIfSceneEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.CommonUtils;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.goods.bean.dto.CategoryAndSceneDto;
import com.jz.manage.moduls.controller.goods.bean.request.AddCateGoryReq;
import com.jz.manage.moduls.controller.goods.bean.request.CategoryAndSceneReq;
import com.jz.manage.moduls.controller.goods.bean.request.CategoryDetailReq;
import com.jz.manage.moduls.controller.goods.bean.request.DataGoodsByCategoryReq;
import com.jz.manage.moduls.entity.DataGoods;
import com.jz.manage.moduls.entity.DataGoodsCategory;
import com.jz.manage.moduls.entity.SceneCategory;
import com.jz.manage.moduls.entity.SceneGoodsCategory;
import com.jz.manage.moduls.mapper.DataGoodsCategoryDao;
import com.jz.manage.moduls.mapper.DataGoodsDao;
import com.jz.manage.moduls.mapper.SceneCategoryDao;
import com.jz.manage.moduls.mapper.SceneGoodsCategoryDao;
import com.jz.manage.moduls.service.DataGoodsCategoryService;
import com.jz.manage.moduls.service.SceneCategoryService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 数据商品分类(行业)(TDataGoodsCategory)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:32
 */
@Service("dataGoodsCategoryService")
public class DataGoodsCategoryServiceImpl implements DataGoodsCategoryService {
    private static final Logger logger = LoggerFactory.getLogger(DataGoodsServiceImpl.class);

    @Autowired
    private DataGoodsCategoryDao categoryDao;

    @Autowired
    private SceneCategoryDao sceneCategoryDao;
    @Autowired
    private SceneGoodsCategoryDao sceneGoodsCategoryDao;

    @Autowired
    private DataGoodsDao dataGoodsDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 查询所有的商品分类
     *
     * @return
     */
    @Override
    public Result<List<DataGoodsCategory>> findByCategory() {
        //1、查出所有分类
        List<DataGoodsCategory> entities = categoryDao.selectByCategory();
        // 找到所有的一级分类
        List<DataGoodsCategory> level1Menus = entities.stream().filter(categoryEntity ->
        categoryEntity.getParentId() == 0 && categoryEntity.getIfChild().equals("N")
        ).map((menu)->{
            menu.setChildren(getChildrens(menu,entities));
            return menu;
        }).collect(Collectors.toList());
        return Result.of_success(ResultMsg.SUCCESS, level1Menus);
    }

    private List<DataGoodsCategory> getChildrens(DataGoodsCategory root,List<DataGoodsCategory> all) {

        List<DataGoodsCategory> children = all.stream().filter(categoryEntity -> {
            return categoryEntity.getParentId() == root.getCategoryId() && categoryEntity.getIfChild().equals("Y");
        }).map(categoryEntity -> {
            //1、找到子菜单,同时递归调用
            categoryEntity.setChildren(getChildrens(categoryEntity, all));
            return categoryEntity;
        }).collect(Collectors.toList());

        return children;
    }

    /**
     * 查询场景分类
     *
     * @return
     */
    @Override
    public Result findByScene() {
        QueryWrapper<SceneCategory> queryWrapper = new QueryWrapper();
        queryWrapper.eq("del_flag", "N");
        List<SceneCategory> list = sceneCategoryDao.selectList(queryWrapper);
        if (list.size() == 0) {
            return Result.of_error(ResultMsg.FAILURE);
        }
        return Result.ok(list);
    }

    /**
     * 添加一级分类
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addCategory(AddCateGoryReq req) throws Exception {
        DataGoodsCategory category = new DataGoodsCategory();
        category.setCategoryName(req.getCategoryName());
        // 添加子级分类
        if (req.getCategoryId() != null) {
            category.setParentId(req.getCategoryId());
            category.setIfChild(YNEnum.YES.getValue());
            QueryWrapper<DataGoodsCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("parent_id", req.getCategoryId());
            queryWrapper.eq("category_name", req.getCategoryName());
            queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
            DataGoodsCategory goodsCategory = categoryDao.selectOne(queryWrapper);
            if (goodsCategory != null) {
                return Result.of_error("同一分类层级已存在相同的分类名称，请填写其他分类名称");
            }
        }else {
            // 添加一级分类
            category.setParentId(0L);
            category.setIfChild(YNEnum.NO.getValue());
            QueryWrapper<DataGoodsCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("parent_id", "0");
            queryWrapper.eq("category_name", req.getCategoryName());
            queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
            DataGoodsCategory goodsCategory = categoryDao.selectOne(queryWrapper);
            if (goodsCategory != null) {
                return Result.of_error("同一分类层级已存在相同的分类名称，请填写其他分类名称");
            }
        }
        category.setCreTime(DateUtils.getToday());
        category.setCrePerson(getUserId()+ "");
        category.setDelFlag(DelFlagEnum.NO.getValue());
        if (categoryDao.insert(category) != 1) {
            throw ResponseException.of_error(ResultMsg.FAILURE);
        }
        logger.info("########################商品分类管理：添加商品分类成功####################");
        // 添加商品-场景关系表
        if (category.getCategoryId() != null && req.getSceneId() != null) {
            SceneGoodsCategory sceneGoodsCategory = new SceneGoodsCategory();
            sceneGoodsCategory.setCategoryId(category.getCategoryId());
            sceneGoodsCategory.setSceneId(req.getSceneId());
            if (sceneGoodsCategoryDao.insert(sceneGoodsCategory) != 1) {
                throw ResponseException.of_error(ResultMsg.FAILURE);
            }
            logger.info("########################商品分类管理：添加商品场景分类成功####################");
        }
        return Result.of_success(ResultMsg.SUCCESS);
    }

    /**
     * 编辑分类
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result editCategory(AddCateGoryReq req) throws Exception {
        // 查询是否重名
        QueryWrapper<DataGoodsCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("category_name", req.getCategoryName());
        queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
        DataGoodsCategory goodsCategory = categoryDao.selectOne(queryWrapper);
        if (goodsCategory != null) {
            return Result.of_error("同一分类层级已存在相同的分类名称，请填写其他分类名称");
        }
        // 修改商品分类
        DataGoodsCategory category = new DataGoodsCategory();
        category.setCategoryId(req.getCategoryId());
        category.setCategoryName(req.getCategoryName());
        category.setUptTime(DateUtils.getToday());
        category.setUptPerson(getUserId()+ "");
        category.setDelFlag(DelFlagEnum.NO.getValue());
        if (categoryDao.updateById(category) != 1) {
            throw ResponseException.of_error(ResultMsg.FAILURE);
        }
        logger.info("########################商品分类管理：编辑商品分类成功####################");
        // 添加商品-场景关系表
        if (category.getCategoryId() != null && req.getSceneId() != null) {
            // 查询关系表
            QueryWrapper query = new QueryWrapper();
            query.eq("scene_id", req.getOldSceneId());
            query.eq("category_id",req.getCategoryId());
            SceneGoodsCategory sceneGoodsCategory = sceneGoodsCategoryDao.selectOne(query);
            sceneGoodsCategory.setCategoryId(category.getCategoryId());
            sceneGoodsCategory.setSceneId(req.getSceneId());
            if (sceneGoodsCategoryDao.updateById(sceneGoodsCategory) != 1) {
                throw ResponseException.of_error(ResultMsg.FAILURE);
            }
            logger.info("########################商品分类管理：编辑商品场景分类成功####################");
        }
        return Result.of_success(ResultMsg.SUCCESS);
    }

    /**
     * 条件查询分类
     *
     * @param search
     * @return
     */
    @Override
    public Result selectByCategory(String search) {
        if (StringUtils.isEmpty(search)) {
            return findByCategory();
        }
        // 模糊查询返回所有id和名字
        QueryWrapper<DataGoodsCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.like("category_name",search);
        queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
        List<DataGoodsCategory> list = categoryDao.selectList(queryWrapper);
        if (list != null) {
             Map<Long, DataGoodsCategory> dicMap = new HashMap<>();
             for (DataGoodsCategory dataGoodsCategory : list) {
                 Long id = dataGoodsCategory.getCategoryId();
                 List<DataGoodsCategory> entities = categoryDao.selectByList(id);
                 entities.forEach(x->{
                     dicMap.put(x.getCategoryId(),x);
                 });
             }
             List<DataGoodsCategory> entities = dicMap.entrySet().parallelStream().map(x->{
                 return x.getValue();
             }).collect(Collectors.toList());
            // 找到所有的一级分类
            List<DataGoodsCategory> level1Menus = entities.stream().filter(categoryEntity ->
                    categoryEntity.getParentId() == 0 && categoryEntity.getIfChild().equals("N")
            ).map((menu)->{
                menu.setChildren(getChildrens(menu,entities));
                return menu;
            }).collect(Collectors.toList());
            return Result.of_success(ResultMsg.SUCCESS, level1Menus);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);


//
//        if (StringUtils.isEmpty(search)) {
//            return findByCategory();
//        }
//        QueryWrapper<DataGoodsCategory> queryWrapper = new QueryWrapper<>();
//        queryWrapper.like("category_name",search);
//        queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
//        List<DataGoodsCategory> list = categoryDao.selectList(queryWrapper);
//        if (list != null) {
//            for (DataGoodsCategory dataGoodsCategory : list) {
//                List<DataGoodsCategory> entities = categoryDao.selectByList(dataGoodsCategory.getCategoryName());
//                // 找到所有的一级分类
//                List<DataGoodsCategory> level1Menus = entities.stream().filter(categoryEntity ->
//                        categoryEntity.getParentId() == 0 && categoryEntity.getIfChild().equals("N")
//                ).map((menu)->{
//                    menu.setChildren(getChildrens(menu,entities));
//                    return menu;
//                }).collect(Collectors.toList());
//                return Result.of_success(ResultMsg.SUCCESS, level1Menus);
//            }
//        }
//        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 查询分类详情
     *
     * @param req
     * @return
     */
    @Override
    public Result getCategoryDetail(CategoryDetailReq req) {
        CategoryAndSceneDto detail = categoryDao.getCategoryDetail(req.getCategoryId());
        if (detail != null) {
            return Result.of_success(detail);
        }
        return Result.of_error(ResultMsg.FAILURE);
    }

    /**
     * 删除分类
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result deleteByCategory(CategoryAndSceneReq req) throws Exception{
        //
        QueryWrapper<DataGoodsCategory> queryWrapperList = new QueryWrapper<>();
        queryWrapperList.eq("parent_id", req.getCategoryId());
        queryWrapperList.eq("del_flag",DelFlagEnum.NO.getValue());
        queryWrapperList.eq("if_child", YNEnum.YES.getValue());
        List<DataGoodsCategory> result = categoryDao.selectList(queryWrapperList);
        if (result.size() != 0) {
            return Result.of_error("该分类下存在子分类不允许删除");
        }
        // 删除category
        DataGoodsCategory category = new DataGoodsCategory();
        category.setCategoryId(req.getCategoryId());
        category.setDelFlag(DelFlagEnum.YES.getValue());
        if (categoryDao.updateById(category) != 1) {
            return Result.of_error(ResultMsg.DELETE_FAIL);
        }
        // 删除商品关联的分类
        QueryWrapper query = new QueryWrapper();
        query.eq("category_id", req.getCategoryId());
        query.eq("del_flag", DelFlagEnum.NO.getValue());
        List<DataGoods> list = dataGoodsDao.selectList(query);
        if (list.size() != 0) {
            dataGoodsDao.updateByCategory(list);
        }
        // 删除category-scene中间表
        if (req.getSceneId() != null) {
            QueryWrapper<SceneGoodsCategory> queryWrapper = new QueryWrapper();
            queryWrapper.eq("category_id", req.getCategoryId());
            queryWrapper.eq("scene_id", req.getSceneId());
            queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
            SceneGoodsCategory sceneGoodsCategory = sceneGoodsCategoryDao.selectOne(queryWrapper);
            if (sceneGoodsCategory != null) {
                sceneGoodsCategory.setDelFlag(DelFlagEnum.YES.getValue());
                if (sceneGoodsCategoryDao.updateById(sceneGoodsCategory) != 1) {
                    return Result.of_error(ResultMsg.DELETE_FAIL);
                }
                return Result.of_success(ResultMsg.DELETE_SUCCESS);
            }
        }
        return Result.of_success(ResultMsg.DELETE_SUCCESS);
    }


    /**
     * 获取用户id
     * @return
     */
    public Long getUserId() {
        SysUserDto sysUserDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return sysUserDto.getUserId();
    }
}