package com.jz.manage.moduls.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.Constants;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.*;
import com.jz.common.enums.auth.AuthModeEnum;
import com.jz.common.enums.auth.AuthTypeEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.*;
import com.jz.manage.moduls.controller.finance.platForm.dto.DepartmentAssetsDto;
import com.jz.manage.moduls.controller.goods.bean.dto.*;
import com.jz.manage.moduls.controller.goods.bean.request.*;
import com.jz.manage.moduls.controller.order.bean.dto.OrderAndAuthorizedDto;
import com.jz.manage.moduls.entity.*;
import com.jz.manage.moduls.mapper.*;
import com.jz.manage.moduls.service.DataGoodsService;
import com.sun.scenario.effect.impl.sw.sse.SSEBlend_SRC_OUTPeer;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.CrossOrigin;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 数据商品(DataGoods)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:30
 */
@Service("dataGoodsService")
@CrossOrigin
public class DataGoodsServiceImpl implements DataGoodsService {
    private static final Logger logger = LoggerFactory.getLogger(DataGoodsServiceImpl.class);
    @Autowired
    private DataGoodsDao dataGoodsDao;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private RoleGoodsDao roleGoodsDao;

    @Autowired
    private OrderPaymentDao orderPaymentDao;

    @Autowired
    private OrderGoodsInfoDao goodsInfoDao;

    @Autowired
    private FinanceCustomerAssetsDao financeCustomerAssetsDao;

    @Autowired
    private FinanceTradeFlowDao financeTradeFlowDao;

    @Autowired
    private DepartmentDao departmentDao;

    //@Autowired
    //private HttpsUtils httpsUtils;

    @Value("${gateway.gatewayGetAuth}")
    private String gatewayGetAuth;


    @Value("${gateway.apigateway}")
    private String apiGateway;

    @Value("${gateway.updateSendStatus}")
    private String updateSendStatus;

    @Value("${gateway.getApiTypeList}")
    private String getApiTypeList;

    @Value("${gateway.getApiDetail}")
    private String getApiDetail;

    @Value("${gateway.listSendDataBank}")
    private String listSendDataBank;

    /**
     * 条件分页查询数据列表
     *
     * @param goodsListRequest
     * @param httpRequest
     * @return
     * @throws Exception
     */
    @Override
    public Result<IPage<DataGoodsListDto>> findList(DataGoodsListRequest goodsListRequest, HttpServletRequest httpRequest) throws Exception {
        IPage<DataGoodsListDto> pageInfoResponse = new Page<>(goodsListRequest.getPageNum(), goodsListRequest.getPageSize());
        QueryWrapper<DataGoodsListDto> queryWrapper = new QueryWrapper<>();
        //数据商品id
        if (StringUtils.isNotBlank(goodsListRequest.getSearch())) {
            queryWrapper.like("t1.data_name", goodsListRequest.getSearch());
        }

        //数据状态：01预售，02在售中，03下架，04上架
        if (StringUtils.isNotBlank(goodsListRequest.getDataStatus())) {
            queryWrapper.eq("t1.data_status", goodsListRequest.getDataStatus());
        }
        if (StringUtils.isNotBlank(goodsListRequest.getCreTime())) {
            String startTime = goodsListRequest.getCreTime().substring(0, 10) + " 00:00:00";
            String endTime = goodsListRequest.getCreTime().substring(11, 21) + " 23:59:59";
            queryWrapper.between("t1.cre_time", startTime, endTime);
        }
        if (StringUtils.isNotBlank(goodsListRequest.getUptTime())) {
            String startTime = goodsListRequest.getUptTime().substring(0, 10) + " 00:00:00";
            String endTime = goodsListRequest.getUptTime().substring(11, 21) + " 23:59:59";
            queryWrapper.between("t1.upt_time", startTime, endTime);
        }
        queryWrapper.eq("t1.del_flag", "N");
        queryWrapper.orderByDesc("t1.cre_time");
        IPage<DataGoodsListDto> list = dataGoodsDao.findList(pageInfoResponse, queryWrapper);
        if (list.getTotal() == 0) {
            return Result.of_error("请制作商品！");
        }
        for (DataGoodsListDto record : list.getRecords()) {
            if (record.getGoodsState().equals("02") && record.getDataStatus().equals("04")) {
                DataGoods dataGoods = new DataGoods();
                dataGoods.setDataGoodsId(record.getDataGoodsId());
                dataGoods.setDataStatus("03");
                dataGoodsDao.updateById(dataGoods);
            }
        }
        return Result.of_success(list);
    }


    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#add(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
     */
//	@Override
//	@Transactional(rollbackFor=Exception.class)
//	public BaseBeanResponse<DataGoods> add(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
//		BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();
//
//		/*$*seDataGoodsIdCodeReplace*$*/
//		//创建人
//		//dataGoods.setCreateUser(getHttpRequestUserId(httpRequest));
//		//创建时间
//		dataGoods.setCreTime(new Date());
//
//		dataGoodsDao.insertSelective(dataGoods);
//
//		baseBeanResponse.setCode(Constants.SUCCESS_CODE);
//		baseBeanResponse.setMessage("新增成功");
//		baseBeanResponse.setData(dataGoods);
//
//		return baseBeanResponse;
//	}

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#edit(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseBeanResponse<DataGoods> edit(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
        BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();

        //修改人
        //dataGoods.setUpdateUser(getHttpRequestUserId(httpRequest));
        //修改时间
        dataGoods.setUptTime(new Date());

        dataGoodsDao.updateByPrimaryKeySelective(dataGoods);

        baseBeanResponse.setCode(Constants.SUCCESS_CODE);
        baseBeanResponse.setMessage("修改成功");
        baseBeanResponse.setData(dataGoods);

        return baseBeanResponse;
    }

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#findById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    public BaseBeanResponse<DataGoodsDto> findById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
            throws Exception {
        BaseBeanResponse<DataGoodsDto> baseBeanResponse = new BaseBeanResponse<>();

        DataGoodsDto DataGoodsDto = dataGoodsDao.findById(dataGoodsRequest.getDataGoodsId());

        baseBeanResponse.setCode(Constants.SUCCESS_CODE);
        baseBeanResponse.setMessage("查询成功");
        baseBeanResponse.setData(DataGoodsDto);

        return baseBeanResponse;
    }

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#deleteById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
            throws Exception {
        BaseResponse baseResponse = new BaseResponse();

        dataGoodsDao.deleteByPrimaryKey(dataGoodsRequest.getDataGoodsId());

        baseResponse.setCode(Constants.SUCCESS_CODE);
        baseResponse.setMessage("删除成功");

        return baseResponse;
    }

    /**
     * 商品上下架
     *
     */
    @Override
    public Result editStatus(DataGoodsEditReq req) throws Exception {
        // 获取用户信息
        SysUserDto sysUserDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        // 获取商品的信息
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("data_goods_id", req.getDataGoodsId());
        queryWrapper.eq("del_flag", "N");
        DataGoods dataGoods = dataGoodsDao.selectOne(queryWrapper);
        if (dataGoods == null) {
            return Result.of_error("查询失败！");
        }
        // 查询当前商品是否在使用
        DataAndOrderDto dataAndOrderDto = orderDao.selectByGoods(req.getDataGoodsId());
        // 上架
        if ("01".equals(req.getStatusId())) {
            // 商品失效
            if (dataAndOrderDto.getOrderState().equals("02")) {
                return Result.of_error("该商品已失效，请修改商品信息或选择其他商品！");
            }
            if (dataGoods.getDataStatus().equals(DataStatusEnum.SJ.getValue())) {
                return Result.of_error("商品已上架，无需上架！");
            }
            dataGoods.setDataGoodsId(req.getDataGoodsId());
            dataGoods.setDataStatus(DataStatusEnum.SJ.getValue());
            dataGoods.setUptPerson(sysUserDto.getUserId() + "");
            dataGoods.setUptTime(DateUtils.getToday());
            if (dataGoodsDao.updateById(dataGoods) != 1) {
                return Result.of_error("上架失败！");
            }
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        // 下架
        if ("02".equals(req.getStatusId())) {
            //商品在使用
            if (dataAndOrderDto.getOrderState().equals("01") && dataAndOrderDto.getOrderTotal() != 0) {
                return Result.of_error("有企业正常使用此商品，暂不允许下架！");
            }
            if (dataGoods.getDataStatus().equals(DataStatusEnum.XJ.getValue())) {
                return Result.of_error("商品已下架，无需下架！");
            }
            dataGoods.setDataGoodsId(req.getDataGoodsId());
            dataGoods.setDataStatus(DataStatusEnum.XJ.getValue());
            dataGoods.setUptPerson(sysUserDto.getUserId() + "");
            dataGoods.setUptTime(DateUtils.getToday());
            if (dataGoodsDao.updateById(dataGoods) != 1) {
                return Result.of_error("下架失败！");
            }
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        return Result.of_error(ResultMsg.UPDATE_FAIL);
    }

    /**
     * 列表查询审核数据商品
     *
     * @param req
     * @param httpRequest
     * @return
     */
    @Override
    public Result<IPage<DataGoodsAuditDto>> findByAudit(DataGoodsAuditListReq req, HttpServletRequest httpRequest) {
        IPage<DataGoodsAuditDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<DataGoodsAuditDto> queryWrapper = new QueryWrapper();
        if (StringUtils.isNotBlank(req.getSearch())) {
            queryWrapper.and(wrapper -> wrapper.like("t1.data_name", req.getSearch())
                    .or().like("t2.category_name", req.getSearch()));
        }
        if (StringUtils.isNotBlank(req.getDataType())) {
            queryWrapper.eq("t1.data_type", req.getDataType());
        }
        if (StringUtils.isNotBlank(req.getAuditStatus())) {
            queryWrapper.eq("t1.audit_status", req.getAuditStatus());
        }
        if (StringUtils.isNotBlank(req.getCreTime())) {
            String startTime = req.getCreTime().substring(0, 10) + " 00:00:00";
            String endTime = req.getCreTime().substring(11, 21) + " 23:59:59";
            queryWrapper.between("t1.cre_time", startTime, endTime);
            queryWrapper.orderByDesc("t1.cre_time");
        }
        if (StringUtils.isNotBlank(req.getUptTime())) {
            String startTime = req.getUptTime().substring(0, 10) + " 00:00:00";
            String endTime = req.getUptTime().substring(11, 21) + " 23:59:59";
            queryWrapper.between("t1.upt_time", startTime, endTime);
            queryWrapper.orderByDesc("t1.upt_time");
        }
        queryWrapper.eq("t1.del_flag", "N");
        IPage<DataGoodsAuditDto> list = dataGoodsDao.findByAudit(page, queryWrapper);
        if (list.getTotal() == 0) {
            return Result.of_error("没有待审核的商品！");
        }
        return Result.of_success(list);
    }

    /**
     * 商品审核管理--商品审核
     *
     * @param req
     * @return
     */
    @Override
    public Result auditDataGoods(DataGoodsAuditReq req) {
        if (req.getDataGoodsId() == null) {
            return Result.error("商品id不能为空");
        }
        if (StringUtils.isBlank(req.getAuditStatus())) {
            return Result.error("审核状态不能为空！");
        }
        QueryWrapper query = new QueryWrapper();
        query.eq("data_goods_id", req.getDataGoodsId());
        query.eq("del_flag", "N");
        DataGoods dataGoods = dataGoodsDao.selectOne(query);
        SysUserDto sysUser = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);

        // 更新信息: 将企业审核状态改为已审核，企业信息更新
        if ("01".equals(req.getAuditStatus())) {
            // 已通过
            dataGoods.setAuditStatus("02");
            dataGoods.setAuditPerson(sysUser.getUserId() + "");
            dataGoods.setAuditTime(DateUtils.getToday());
            dataGoods.setUptTime(DateUtils.getToday());
            dataGoods.setUptPerson(sysUser.getUserId() + "");
            dataGoods.setDataStatus(DataStatusEnum.SJ.getValue());
            dataGoodsDao.updateById(dataGoods);
            return Result.of_success("商品审核已通过!");
        }
        if ("02".equals(req.getAuditStatus())) {
            // 未通过
            dataGoods.setAuditStatus("03");
            dataGoods.setAuditPerson(sysUser.getUserId() + "");
            dataGoods.setAuditTime(DateUtils.getToday());
            dataGoods.setUptTime(DateUtils.getToday());
            dataGoods.setUptPerson(sysUser.getUserId() + "");
            dataGoods.setDataStatus(DataStatusEnum.XJ.getValue());
            dataGoods.setRejectReason(req.getRejectReason());
            dataGoodsDao.updateById(dataGoods);
            return Result.of_success("商品审核未通过!");
        }
        return Result.of_error("审核失败!");
    }

    /**
     * 获取api列表
     *
     * @return
     */
    @Override
    public Result getAPIList(SendDataReq req) {
        String s = HttpsUtilsTest.submitPost(listSendDataBank, JSON.toJSONString(req));
        JSONObject params = JSONObject.parseObject(s);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.of_error("获取api列表失败！");
        }
        JSONObject dataParams = JSONObject.parseObject(params.getString("data"));

        JSONArray array = dataParams.getJSONArray("records");
        List<ApiDataListDto> list = JSONObject.parseArray(array.toJSONString(), ApiDataListDto.class);
//        if (list != null) {
//            List<ApiDataGoodsListDto> apiDataGoodsListDtos = dataGoodsDao.selectByApiDataGoodsList(list);
//            Map<String, List<ApiDataGoodsListDto>> collect = apiDataGoodsListDtos.stream().collect(Collectors.groupingBy(ApiDataGoodsListDto::getApiKey));
//            for (Map.Entry<String, List<ApiDataGoodsListDto>> entry : collect.entrySet()) {
//                for (ApiDataListDto apiDataListDto : list) {
//                    if (entry.getKey().equals(apiDataListDto.getApiKey())) {
//                        apiDataListDto.setChildren(entry.getValue());
//                    }
//                }
//            }
//        }
//        return null;
        return Result.of_success(list);
    }

    /**
     * 获取api详情
     *
     * @param req
     * @return
     */
    @Override
    public Result getApiInterfaceDetail(ApiInterfaceDetailReq req) {
        String s = HttpsUtilsTest.submitPost(getApiDetail, JSON.toJSONString(req));
        JSONObject params = JSONObject.parseObject(s);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.of_error("获取api详情失败！");
        }
        DataDetailsDto dataDetailsDto = JSONObject.parseObject(params.getString("data"), DataDetailsDto.class);

        return Result.of_success(dataDetailsDto);
    }

    /**
     * 制作商品
     *
     * @param req
     * @return
     */
    @Override
    public Result makeByGoods(DataApiUpReq req) throws Exception {
        // 获取当前用户
        SysUserDto user = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        if (user == null) {
            return Result.of_error("当前用户不存在！请登录！");
        }
        // 保存商品
        DataGoods dataGoods = new DataGoods();
        dataGoods.setDataStatus(DataStatusEnum.XJ.getValue());
        dataGoods.setAuditStatus(AuditStatusEnum.YSH.getValue());
        dataGoods.setAuditPerson(user.getUserId() + "");
        dataGoods.setAuditTime(DateUtils.getToday());
        dataGoods.setCreTime(DateUtils.getToday());
        dataGoods.setCrePerson(user.getUserId() + "");
        dataGoods.setUserId(user.getUserId());
        dataGoods.setGoodsLimited(DateUtils.getLimitedTime(req.getGoodsLimited()));
        dataGoods.setDelFlag("N");
        // 校验商品价格
        if (req.getSecondType() != null) {
            dataGoods.setDataPrice(req.getSecondType());
        } else if (req.getMonthType() != null) {
            dataGoods.setDataPrice(req.getMonthType());
        } else if (req.getSeasonType() != null) {
            dataGoods.setDataPrice(req.getSeasonType());
        } else {
            dataGoods.setDataPrice(req.getYearType());
        }
        BeanUtils.copyProperties(req, dataGoods);
        if (dataGoodsDao.insert(dataGoods) != 1) {
            throw ResponseException.of_error("制作商品失败！");
        }
        return Result.ok();
    }

    /**
     * 浏览授权 TODO 待优化
     *
     * @param roleReq
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result browseByAuthorization(DataGoodsByRoleReq roleReq) {
        RoleGoods role = roleGoodsDao.findByRole(roleReq.getDataGoodsId());
        RoleGoods roleGoods = new RoleGoods();
        // 第一次授权
        if (role == null && roleReq.getLogOutUser().equals("01")) {
            roleGoods.setDataGoodsId(roleReq.getDataGoodsId());
            roleGoods.setRoleId(Long.valueOf(DataGoodsByRoleEnum.logOutUser.getValue()));
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            if (roleGoodsDao.insert(roleGoods) != 1) {
                throw ResponseException.of_error("未登录用户授权失败！");
            }
            // 关闭授权
        } else if (roleReq.getLogOutUser().equals("02")) {
            roleGoods.setDelFlag(DelFlagEnum.YES.getValue());
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", roleReq.getDataGoodsId());
            queryWrapper.eq("role_id", Long.valueOf(DataGoodsByRoleEnum.logOutUser.getValue()));
            if (roleGoodsDao.update(roleGoods, queryWrapper) != 1) {
                return Result.of_error(ResultMsg.FAILURE);
            }
            // 开启授权
        }else if (roleReq.getLogOutUser().equals("01")) {
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", roleReq.getDataGoodsId());
            queryWrapper.eq("role_id", Long.valueOf(DataGoodsByRoleEnum.logOutUser.getValue()));
            if (roleGoodsDao.update(roleGoods, queryWrapper) != 1) {
                return Result.of_error(ResultMsg.FAILURE);
            }
        }

        if (role == null  && roleReq.getUnverifiedUser().equals("01")) {
            roleGoods.setDataGoodsId(roleReq.getDataGoodsId());
            roleGoods.setRoleId(Long.valueOf(DataGoodsByRoleEnum.unverifiedUser.getValue()));
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            if (roleGoodsDao.insert(roleGoods) != 1) {
                throw ResponseException.of_error("已登录未认证用户授权失败！");
            }
        } else if (roleReq.getUnverifiedUser().equals("02")) {
            roleGoods.setDelFlag(DelFlagEnum.YES.getValue());
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", roleReq.getDataGoodsId());
            queryWrapper.eq("role_id", Long.valueOf(DataGoodsByRoleEnum.unverifiedUser.getValue()));
            if (roleGoodsDao.update(roleGoods, queryWrapper) != 1) {
                return Result.of_error(ResultMsg.FAILURE);
            }
        }else if (roleReq.getUnverifiedUser().equals("01")) {
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", roleReq.getDataGoodsId());
            queryWrapper.eq("role_id", Long.valueOf(DataGoodsByRoleEnum.unverifiedUser.getValue()));
            if (roleGoodsDao.update(roleGoods, queryWrapper) != 1) {
                return Result.of_error(ResultMsg.FAILURE);
            }
        }

        if (role == null  && roleReq.getAuthenticatedUser().equals("01")) {
            roleGoods.setDataGoodsId(roleReq.getDataGoodsId());
            roleGoods.setRoleId(Long.valueOf(DataGoodsByRoleEnum.authenticatedUser.getValue()));
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            if (roleGoodsDao.insert(roleGoods) != 1) {
                throw ResponseException.of_error("认证用户授权失败！");
            }
        } else if (roleReq.getAuthenticatedUser().equals("02")) {
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", roleReq.getDataGoodsId());
            queryWrapper.eq("role_id", Long.valueOf(DataGoodsByRoleEnum.authenticatedUser.getValue()));
            if (roleGoodsDao.update(roleGoods, queryWrapper) != 1) {
                return Result.of_error(ResultMsg.FAILURE);
            }
        }
        return Result.ok("授权成功！");
    }

    /**
     * 查询授权
     *
     * @param dataGoodsId
     * @return
     */
    @Override
    public Result findByAuthorization(Long dataGoodsId) {
        if (dataGoodsId != null) {
            QueryWrapper<RoleGoods> queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", dataGoodsId);
            queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
            List<RoleGoods> roleGoods = roleGoodsDao.selectList(queryWrapper);
            if (roleGoods.size() != 0) {
                return Result.ok(roleGoods);
            }
            return Result.ok("用户未授权！");
        }
        return Result.of_error("查询失败！");
    }

    /**
     * 查询商品详情
     *
     * @param id
     * @return
     */
    @Override
    public Result selectById(Long id) {
        QueryWrapper<DataGoods> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("data_goods_id",id);
        queryWrapper.eq("data_status", DataStatusEnum.SJ.getValue());
        queryWrapper.last("and (goods_limited >= DATE_FORMAT(NOW(), '%Y-%m-%d'))");
        DataGoods dataGoods = dataGoodsDao.selectOne(queryWrapper);
        if (dataGoods != null) {
            return Result.ok(dataGoods);
        }
        return Result.of_error("商品未上架，无效!");
    }

    /**
     * 使用授权
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result useByAuthorization(GoodsByAuthorizationReq req) throws Exception{
        if (req.getDepartmentId() == null || req.getAssetsId() == null) {
            return Result.of_error("用户没关联企业，支付失败!");
        }
        // 校验商品是否已授权
        OrderAndAuthorizedDto dto = orderDao.isAuthorization(req.getDepartmentId(),req.getDataGoodsId());
        if (dto != null && dto.getGoodsState().equals("01")) {
            return Result.of_error("该企业已经有该商品的使用授权，请选择其他企业进行授权!");
        }
        if (dto == null) {
            // 获取授权
            Result authorization = getAuthorization(req);
            if (!authorization.isSuccess()) {
                throw ResponseException.of_error("获取授权失败！");
            }
            // 新增订单关系表：订单表，订单支付表，订单商品表
            Result result = addOrder(req);
            if (!result.isSuccess()) {
                throw ResponseException.of_error("新增订单关系表失败！");
            }
            Order order = (Order) result.getData();
            // 修改企业资产关系表：资产表，新增记录
            Result params = uptAssets(req, order);
            if (!params.isSuccess()) {
                throw ResponseException.of_error("修改企业资产关系表失败！");
            }
            Map<String, String> map = (Map) params.getData();
            return Result.of_success(ResultMsg.SUCCESS,map);
        }
        return Result.of_error(ResultMsg.FAILURE);
    }

    /**
     * 搜索企业
     * @param req
     * @return
     */
    @Override
    public Result findByDepartment(DepartmentGoodsReq req) {
        QueryWrapper<DepartmentAssetsDto> query = new QueryWrapper();
        // 使用授权时搜索企业
        if (StringUtils.isNotEmpty(req.getSearch())) {
            query.like("t1.department_name", req.getSearch());
        }
        // 续费时搜索企业
        if (req.getDepartmentId() != null ) {
            query.eq("t1.department_id", req.getDepartmentId());
            query.eq("t6.data_goods_id", req.getDataGoodsId());
        }
        query.eq("t1.del_flag", DelFlagEnum.NO.getValue());
        query.groupBy("t1.department_id");
        List<DepartmentAssetsDto> department = departmentDao.selectByDepartment(query);
        if (department.size() != 0) {
            return Result.of_success(department);
        }
        return Result.of_success(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 获取制作的商品
     *
     * @param req
     * @return
     */
    @Override
    public Result getDataGoodsNewsList(DataGoodsByMakeReq req) {
        IPage<DataGoods> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<DataGoods> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getDataName())) {
            queryWrapper.like("data_name", req.getDataName());
        }
        if (req.getDataStatus() != null) {
            queryWrapper.eq("data_status", req.getDataStatus());
        }
        queryWrapper.eq("api_key", req.getApiKey());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        IPage<DataGoods> dataGoodsIPage = dataGoodsDao.selectPage(page, queryWrapper);
        if (dataGoodsIPage != null) {
            return Result.ok(dataGoodsIPage);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }


    /**
     * 获取数据类型
     *
     * @return
     */
    @Override
    public Result findByDataTypeList() {
        Map<String, String> map = new HashMap<>();
        map.put("parentKey", "10006");
        String get = HttpsUtilsTest.doGet(getApiTypeList, map);
        JSONObject params = JSONObject.parseObject(get);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.of_error("获取api详情失败！");
        }
        return Result.ok(params.get("data"));
    }

    /**
     * 退回数据到dmp
     *
     * @param req
     * @return
     */
    @Override
    public Result updateSendStatus(ApiInterfaceDetailReq req) {
        QueryWrapper<DataGoods> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("api_key", req.getApiKey());
        queryWrapper.eq("data_status", DataStatusEnum.SJ.getValue());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        List<DataGoods> dataGoods = dataGoodsDao.selectList(queryWrapper);
        if (dataGoods.size() != 0) {
            return Result.of_error("已被制作成商品或者商品未下架！");
        }
        Map<String, String> map = new HashMap<>();
        map.put("apiKey", req.getApiKey());
        String get = HttpsUtilsTest.doGet(updateSendStatus, map);
        JSONObject params = JSONObject.parseObject(get);
        if (!params.get("code").equals(200)) {
            return Result.of_error("获取api详情失败！");
        }
        return Result.of_success(ResultMsg.SUCCESS,params.get("data"));
    }


    /**
     * 浏览授权优化
     */
    public Result addByAuthorization(RoleGoods role,DataGoodsByRoleReq roleReq, String dataGoodsByRoleEnum) {
        RoleGoods roleGoods = new RoleGoods();
        if (role == null && roleReq.getLogOutUser().equals("01")) {
            roleGoods.setDataGoodsId(roleReq.getDataGoodsId());
            roleGoods.setRoleId(Long.valueOf(dataGoodsByRoleEnum));
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            if (roleGoodsDao.insert(roleGoods) != 1) {
                throw ResponseException.of_error("未登录用户授权失败！");
            }
            // 关闭授权
        } else if (roleReq.getLogOutUser().equals("02")) {
            roleGoods.setDelFlag(DelFlagEnum.YES.getValue());
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", roleReq.getDataGoodsId());
            queryWrapper.eq("role_id", Long.valueOf(dataGoodsByRoleEnum));
            if (roleGoodsDao.update(roleGoods, queryWrapper) != 1) {
                return Result.of_error(ResultMsg.FAILURE);
            }
            // 开启授权
        }else if (roleReq.getLogOutUser().equals("01")) {
            roleGoods.setDelFlag(DelFlagEnum.NO.getValue());
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("data_goods_id", roleReq.getDataGoodsId());
            queryWrapper.eq("role_id", Long.valueOf(dataGoodsByRoleEnum));
            if (roleGoodsDao.update(roleGoods, queryWrapper) != 1) {
                return Result.of_error(ResultMsg.FAILURE);
            }
        }
        return Result.ok();
    }

    /**
     * 制作有效时间
     * @param req
     * @return
     */
    public static Date getTime(GoodsByAuthorizationReq req) {
        String purchaseMethod = req.getPurchaseMethod();
        Date now = DateUtils.getToday();
        if ("01".equals(purchaseMethod)) {
            Date nowAddYear = DateUtils.getAfterYear(now);
            return nowAddYear;
        }else if ("02".equals(purchaseMethod)){
            Date nowAddSeason = DateUtils.getAfterSeason(now);
            return nowAddSeason;
        }else if("03".equals(purchaseMethod)) {
            Date nowAddMonth = DateUtils.getAfterMonth(now);
            return nowAddMonth;
        }else {
            return null;
        }
    }

    /**
     * 授权: 获取盐值和授权码
     */
    public Result getAuthorization(GoodsByAuthorizationReq req) throws Exception{
        // 年季月次 商品
        AuthMallUserApiReq apiReq = new AuthMallUserApiReq();
        if ("04".equals(req.getPurchaseMethod())) {
            apiReq.setAuthMode(AuthModeEnum.POWER_CALL_MODE.name());
            apiReq.setValidStartTime(req.getTakeEffectTime());
            apiReq.setValidEndTime(req.getInvalidTime());
        }else {
            apiReq.setAuthMode(AuthModeEnum.RECORD_TIME_MODE.name());
            apiReq.setValidStartTime(req.getTakeEffectTime());
            apiReq.setValidEndTime(req.getInvalidTime());
        }
        apiReq.setApiKey(req.getApiKey());
        apiReq.setAuthType(AuthTypeEnum.DATA_BANK_AUTH.name());
        apiReq.setOrgCode("000002"); // TODO
        apiReq.setCreateUser(getUserId() +"");

        String post = HttpsUtilsTest.submitPost(gatewayGetAuth, JSON.toJSONString(apiReq));
        JSONObject params = JSONObject.parseObject(post);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))){
            throw ResponseException.of_error("授权失败！");
        }
        logger.info("###########################购买商品：授权成功！##############################");
        // 获取到盐值和token
        String data = params.getString("data");
        JSONObject parse = JSONObject.parseObject(data);
        String authCode = parse.getString("authCode");
        String salt = parse.getString("salt");
        if (StringUtils.isEmpty(authCode) || StringUtils.isEmpty(salt)) {
            return Result.of_error("授权失败，未获取到授权码和盐值！");
        }
        Map<String, String> map = new HashMap();
        map.put("authCode",authCode);
        map.put("salt",salt);
        return Result.ok(map);
    }

    /**
     * 新增订单关系表：订单表，订单支付表，订单商品表
     */
    public Result addOrder(GoodsByAuthorizationReq payReq) throws ResponseException{
        // 新增订单主表
        Order order = new Order();
        order.setCustomerId(payReq.getCustomerId());
        order.setDepartmentId(payReq.getDepartmentId());
        order.setOrderMoney(payReq.getDataPrice());
        order.setOrderStatus(OrderStatusEnum.YZF.getValue());
        order.setPaymentMoney(payReq.getDataPrice());
        order.setOrderTime(DateUtils.getToday());
        order.setPaymentTime(order.getOrderTime());
        order.setPaymentMethod(PaymentMethodEnum.YE.getValue());
        order.setSellerId(getUserId());  //
//        order.setDistrictMoney(); //TODO  优惠金额
        order.setTakeEffectTime(order.getOrderTime());
        order.setInvalidTime(payReq.getInvalidTime());
        order.setApiKey(payReq.getApiKey());
        //处理生效期
//        Date time = getTime(payReq);
//        if (time != null) {
//            order.setInvalidTime(time);
//        }
        order.setPurchaseMethod(payReq.getPurchaseMethod());
        order.setPriceType(payReq.getPriceType());
        order.setCrePerson(payReq.getDepartmentId() + "");
        order.setCreTime(DateUtils.getToday());
        order.setDelFlag(DelFlagEnum.NO.getValue());
        if (orderDao.insert(order) != 1) {
            throw ResponseException.of_error("新增订单失败！");
        }
        logger.info("###########################购买商品：新增订单成功！##############################");
        // 新增订单支付表/订单商品表
        if (order.getOrderId() != null) {
            OrderPayment orderPayment = new OrderPayment();
            orderPayment.setOrderId(order.getOrderId());
            orderPayment.setSellerId(getUserId()); // TODO
            orderPayment.setPaymentMoney(payReq.getDataPrice());
            orderPayment.setPaymentTime(DateUtils.getToday());
            orderPayment.setPaymentMethod(PaymentMethodEnum.YE.getValue());
//            orderPayment.setDistrictMoney(); //TODO
            orderPayment.setPaymentStatus(order.getOrderStatus());
            orderPayment.setPurchaseMethod(payReq.getPurchaseMethod());
            orderPayment.setCreTime(DateUtils.getToday());
            orderPayment.setCrePerson(payReq.getDepartmentId()+"");
            orderPayment.setDelFlag(DelFlagEnum.NO.getValue());
            if (orderPaymentDao.insert(orderPayment) != 1) {
                throw ResponseException.of_error("新增订单支付表失败！");
            }
            logger.info("###########################购买商品：新增订单支付表成功！##############################");
            // 订单商品表
            OrderGoodsInfo goodsInfo = new OrderGoodsInfo();
            goodsInfo.setOrderId(order.getOrderId());
            goodsInfo.setDataGoodsId(payReq.getDataGoodsId());
            goodsInfo.setCreTime(DateUtils.getToday());
            goodsInfo.setCrePerson(payReq.getDepartmentId()+"");
            goodsInfo.setDelFlag(DelFlagEnum.NO.getValue());
            // 新增
            if (goodsInfoDao.insert(goodsInfo) != 1) {
                throw ResponseException.of_error("新增订单商品表失败！");
            }
            logger.info("###########################购买商品：新增订单商品表成功！##############################");
        }
        return Result.ok(order);
    }

    /**
     * 修改企业资产关系表：资产表，新增记录
     */
    public Result uptAssets(GoodsByAuthorizationReq req, Order order) throws Exception{
        // 获取企业资产信息
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("assets_id", req.getAssetsId());
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        FinanceCustomerAssets assets = financeCustomerAssetsDao.selectOne(queryWrapper);
        // 可用金额减去支付金额
        BigDecimal bigDecimal = new BigDecimal(assets.getUseMoney()+ "");
        BigDecimal bigDecima2 = new BigDecimal(req.getDataPrice()+ "");
//        if (bigDecimal.compareTo(bigDecima2) == -1){
//            return Result.error("金额不足！请充值！");
//        }
//        BigDecimal subtract = bigDecimal.subtract(bigDecima2);
        BigDecimal subtract = BigDecimal.ZERO;
        // 修改金额
        assets.setUseMoney(subtract);
        assets.setTotalMoney(subtract);
        assets.setUptTime(DateUtils.getToday());
        assets.setUptPerson(req.getDepartmentId()+ "");
        if (financeCustomerAssetsDao.updateById(assets) != 1) {
            throw ResponseException.of_error("修改资产金额失败！");
        }
        logger.info("###########################购买商品：修改资产金额成功！##############################");
        // 授权：获取到盐值和token，更新订单表
        Result authorization = getAuthorization(req);
        Map<String, String> map= (Map) authorization.getData();
        order.setOrderId(order.getOrderId());
        order.setGoodsToken(map.get("authCode"));
        order.setSaltValue(map.get("salt"));
        order.setUptTime(DateUtils.getToday());
        order.setOrderNumber(NumberUtils.getOrderNumber(req.getDepartmentId()+""));
        order.setUptPerson(req.getDepartmentId() + "");
        if (orderDao.updateById(order) != 1) {
            throw ResponseException.of_error("修改订单盐值失败！");
        }
        logger.info("###########################购买商品：修改订单盐值成功！##############################");
        // 新增记录表
        Long assetsId = assets.getAssetsId();
        if (assetsId!= null) {
            FinanceTradeFlow flow = new FinanceTradeFlow();
            flow.setAssetsId(assetsId);
            flow.setOrderId(order.getOrderId());
            flow.setTradeFlowNumber(NumberUtils.getDealNum());// 交易流水编号
            flow.setTradeMoney(req.getDataPrice());
            flow.setTradeType(TradeFlowEnum.FK.getValue());
            flow.setCreTime(DateUtils.getToday());
            flow.setCrePerson(req.getDepartmentId() + "");
            flow.setTradeFlowNumber(NumberUtils.getDealNum());
            flow.setDelFlag(DelFlagEnum.NO.getValue());
            if (financeTradeFlowDao.insert(flow) != 1) {
                throw ResponseException.of_error("新增记录表失败！");
            }
            logger.info("###########################购买商品：新增记录表成功！##############################");
        }
        return Result.ok(map);
    }

    /**
     * 获取当前用户
     * @return
     */
    public Long getUserId() {
        SysUserDto sysUserDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return sysUserDto.getUserId();
    }

    /**
     * 自营数据-上传api
     *
     * @param dataApiUpReq
     */
//    @Override
//    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
//    public Result uploadApi(DataApiUpReq dataApiUpReq) throws Exception  {
//        // 获取当前用户
//        SysUserDto user = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
//
//        if (dataApiUpReq == null) {
//            return Result.error("上传失败！");
//        }
//        if (!StringUtils.isNotEmpty(dataApiUpReq.getReturnType())) {
//            return Result.error("输出类型不能为空");
//        }
//        if (!StringUtils.isNotEmpty(dataApiUpReq.getJoinType())) {
//            return Result.error("接入类型不存在！");
//        }
//        if (!StringUtils.isNotEmpty(dataApiUpReq.getApiProtocl())) {
//            return Result.error("传输方式不能为空！");
//        }
//        if (!StringUtils.isNotEmpty(dataApiUpReq.getEncryMode())) {
//            return Result.error("加密方式不能为空！");
//        }
//        // 调用gateway获取apiKey
//        MakeDataBankApiReq apiReq = new MakeDataBankApiReq();
//        apiReq.setApiType(dataApiUpReq.getApiType());
//        apiReq.setApiDesc(dataApiUpReq.getApiDesc());
//        apiReq.setApiProtocl(dataApiUpReq.getApiProtocl());
//        apiReq.setSignType(dataApiUpReq.getEncryMode());
//        apiReq.setTargetUrl(dataApiUpReq.getTargetUrl());
//        apiReq.setApiFunction(dataApiUpReq.getApiFunction());
//        apiReq.setApiUrl(dataApiUpReq.getApiUrl());
//        apiReq.setJoinType(dataApiUpReq.getJoinType());
//        apiReq.setOutputType(dataApiUpReq.getReturnType());
//        apiReq.setPage(false); //TODO
//        apiReq.setReqType("DAY"); // TODO
//        apiReq.setFileId(1L);
////        apiReq.setTimeout(dataApiUpReq.getOverTime() + "");
//        for (DataGoodsApiParams params : dataApiUpReq.getInboxParam()) {
//            apiReq.setInboxParam(JSON.toJSONString(params));
//        }
//        for (DataGoodsApiParams reqHeader : dataApiUpReq.getReqHeaders()) {
//            apiReq.setReqHeaders(JSON.toJSONString(reqHeader));
//        }
//        for (DataGoodsApiParams params : dataApiUpReq.getRequestParam()) {
//            apiReq.setRequestParam(JSON.toJSONString(params));
//        }
//        for (DataGoodsApiParams params : dataApiUpReq.getResponseParam()) {
//            apiReq.setResponseParam(JSON.toJSONString(params));
//        }
//        for (DataGoodsApiParams params : dataApiUpReq.getRespCode()) {
//            apiReq.setResponseParam(JSON.toJSONString(params));
//        }
//        apiReq.setApiExample(dataApiUpReq.getReturnDataExample());
//
//
//        String post = HttpsUtilsTest.submitPost(apiGateway, JSON.toJSONString(apiReq));
//        JSONObject result = JSON.parseObject(post);
//        if (!result.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(result.get("data"))){
//            return Result.of_error("制作失败！");
//        }
//        //封装参数
//        DataGoods dataGoods = new DataGoods();
//        dataGoods.setApiOpenid(dataApiUpReq.getApiOpenId());
//        dataGoods.setCreTime(DateUtils.getToday());
//        dataGoods.setCrePerson(user.getUserId() + "");
//        dataGoods.setAuditTime(DateUtils.getToday());
//        dataGoods.setAuditPerson(user.getUserId() + "");
//        dataGoods.setDataType(dataApiUpReq.getApiType());
//        dataGoods.setUserId(user.getUserId());
//        dataGoods.setDataPicture(dataApiUpReq.getDataPicture());
//        dataGoods.setAuditStatus("02");
//        dataGoods.setDataStatus("02");
//        dataGoods.setDelFlag("N");
//        // 校验商品价格
//        if (!org.springframework.util.StringUtils.isEmpty(dataApiUpReq.getSecondType())) {
//            dataGoods.setDataPrice(dataApiUpReq.getSecondType());
//        }else if (!org.springframework.util.StringUtils.isEmpty(dataApiUpReq.getMonthType())) {
//            dataGoods.setDataPrice(dataApiUpReq.getMonthType());
//        }else if (!org.springframework.util.StringUtils.isEmpty(dataApiUpReq.getSeasonType())) {
//            dataGoods.setDataPrice(dataApiUpReq.getSeasonType());
//        }else {
//            dataGoods.setDataPrice(dataApiUpReq.getYearType());
//        }
//        BeanUtils.copyProperties(dataApiUpReq, dataGoods);
//        dataGoodsDao.insert(dataGoods);
//
//        // 赋值api
//        Long dataGoodsId = dataGoods.getDataGoodsId();
//        if (dataGoodsId != null) {
//            DataGoodsApi dataGoodsApi = new DataGoodsApi();
//
//            dataGoodsApi.setDataGoodsId(dataGoodsId);
//            dataGoodsApi.setApiType(dataApiUpReq.getApiType());
//            dataGoodsApi.setCreTime(DateUtils.getToday());
//            dataGoodsApi.setCrePerson(user.getUserId() + "");
//            dataGoodsApi.setEncryMode("MD5");
//            dataGoodsApi.setTransMode("http");
//            dataGoodsApi.setVersionNumber("v1.0");
//            dataGoodsApi.setDelFlag("N");
//            dataGoodsApi.setApiKey(result.get("data").toString());
//            BeanUtils.copyProperties(dataApiUpReq, dataGoodsApi);
//            tDataGoodsApiDao.insert(dataGoodsApi);
//
//            // 赋值params
//            Long goodsApi = dataGoodsApi.getGoodsApi();
//            if (goodsApi != null) {
//                // 公共参数
//                List<DataGoodsApiParams> inboxParam = dataApiUpReq.getInboxParam();
//                if (inboxParam != null || inboxParam.size() != 0) {
//                    for (DataGoodsApiParams params : inboxParam) {
//                        params.setGoodsApi(goodsApi);
//                        params.setParamsDiff("01");
//                        params.setDelFlag("N");
//                        params.setCreTime(DateUtils.getToday());
//                        params.setCrePerson(user.getUserId() + "");
//                        BeanUtils.copyProperties(dataApiUpReq, params);
//                        apiParamsDao.insert(params);
//                    }
//                }
//                // 请求参数
//                List<DataGoodsApiParams> requestParam = dataApiUpReq.getRequestParam();
//                if (requestParam != null || requestParam.size() != 0) {
//                    for (DataGoodsApiParams params : requestParam) {
//                        params.setGoodsApi(goodsApi);
//                        params.setParamsDiff("02");
//                        params.setDelFlag("N");
//                        params.setCreTime(DateUtils.getToday());
//                        params.setCrePerson(user.getUserId() + "");
//                        BeanUtils.copyProperties(dataApiUpReq, params);
//                        apiParamsDao.insert(params);
//                    }
//                }
//                // 响应参数
//                List<DataGoodsApiParams> responseParam = dataApiUpReq.getResponseParam();
//                if (responseParam != null || responseParam.size() != 0) {
//                    for (DataGoodsApiParams params : responseParam) {
//                        params.setGoodsApi(goodsApi);
//                        params.setParamsDiff("03");
//                        params.setDelFlag("N");
//                        params.setCreTime(DateUtils.getToday());
//                        params.setCrePerson(user.getUserId() + "");
//                        BeanUtils.copyProperties(dataApiUpReq, params);
//                        apiParamsDao.insert(params);
//                    }
//                }
//                // 请求头参数
//                List<DataGoodsApiParams> paramsList = dataApiUpReq.getReqHeaders();
//                if (paramsList != null || paramsList.size() != 0) {
//                    for (DataGoodsApiParams params : paramsList) {
//                        params.setGoodsApi(goodsApi);
//                        params.setParamsDiff("04");
//                        params.setDelFlag("N");
//                        params.setCreTime(DateUtils.getToday());
//                        params.setCrePerson(user.getUserId() + "");
//                        BeanUtils.copyProperties(dataApiUpReq, params);
//                        apiParamsDao.insert(params);
//                    }
//                }
//                // 状态码参数
//                List<DataGoodsApiParams> respCode = dataApiUpReq.getRespCode();
//                if (respCode != null || respCode.size() != 0) {
//                    for (DataGoodsApiParams params : paramsList) {
//                        params.setGoodsApi(goodsApi);
//                        params.setParamsDiff("05");
//                        params.setDelFlag("N");
//                        params.setCreTime(DateUtils.getToday());
//                        params.setCrePerson(user.getUserId() + "");
//                        BeanUtils.copyProperties(dataApiUpReq, params);
//                        apiParamsDao.insert(params);
//                    }
//                }
//            }
//        }
//        return Result.of_success(ResultMsg.SUCCESS);
//    }
}