package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.goods.bean.request.DemandAuditReq;
import com.jz.manage.moduls.controller.goods.bean.request.DemandListReq;
import com.jz.manage.moduls.entity.Demand;
import com.jz.manage.moduls.mapper.DemandDao;
import com.jz.manage.moduls.service.DemandService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

/**
 * 用户需求定制申请(TDemand)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:34
 */
@Service("demandService")
public class DemandServiceImpl implements DemandService {
    @Autowired
    private DemandDao tDemandDao;

    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 条件查询需求
     *
     * @param req
     * @return
     */
    @Override
    public Result<IPage<Demand>> findList(DemandListReq req) {
        IPage<Demand> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<Demand> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(req.getSearch())) {
            queryWrapper.and(s -> s.like("demand_name",req.getSearch())
                    .or().like("demand_describe", req.getSearch()));
        }
        if (StringUtils.isNotEmpty(req.getAuditStatus())) {
            queryWrapper.eq("audit_status", req.getAuditStatus());
        }
        if (StringUtils.isNotBlank(req.getAuditTime())) {
            String startTime = req.getAuditTime().substring(0, 10) + " 00:00:00";
            String endTime = req.getAuditTime().substring(11, 21) + " 23:59:59";
            queryWrapper.between("audit_time", startTime, endTime);
        }
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        queryWrapper.orderByDesc("cre_time");
        IPage<Demand> list = tDemandDao.selectPage(page, queryWrapper);
        if (list.getTotal() != 0) {
            return Result.of_success(list);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }
    /**
     * 查询需求详情
     *
     * @param demandId
     * @return
     */
    @Override
    public Result findByDemand(Long demandId) {
        QueryWrapper<Demand> query = new QueryWrapper<>();
        query.eq("demand_id", demandId);
        query.eq("del_flag", DelFlagEnum.NO.getValue());
        Demand demand = tDemandDao.selectOne(query);
        if (demand == null) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        return Result.of_success(demand);
    }

    /**
     * 审核需求
     *
     * @param req
     * @return
     */
    @Override
    public Result auditDemand(DemandAuditReq req) {
        // 查询审核的需求
        QueryWrapper<Demand> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("demand_id",req.getDemandId());
        queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
        Demand demand = tDemandDao.selectOne(queryWrapper);
        if (demand == null) {
            return Result.of_error(ResultMsg.AUDIT_FAIL);
        }
        if ("01".equals(demand.getAuditStatus())) {
            // 审核通过
            if ("01".equals(req.getAuditStatus())) {
                demand.setAuditStatus(AuditStatusEnum.YSH.getValue());
            }
            // 审核未通过
            if ("02".equals(req.getAuditStatus())) {
                demand.setAuditStatus(AuditStatusEnum.WTG.getValue());
            }
            demand.setRejectReason(req.getRejectReason());
            demand.setAuditPerson(userDto().getUserId() + "");
            demand.setAuditTime(DateUtils.getToday());
            if (tDemandDao.updateById(demand) != 1) {
                return Result.of_error(ResultMsg.AUDIT_FAIL);
            }
            return Result.of_success(ResultMsg.AUDIT_SUCCESS);
        }
        return Result.of_error("需求已审核");
    }

    public SysUserDto userDto() {
        SysUserDto userDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return userDto;
    }
}