package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.AuditStatusEnum;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.enums.TradeFlowEnum;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.NumberUtils;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.finance.platForm.dto.*;
import com.jz.manage.moduls.controller.finance.platForm.req.*;
import com.jz.manage.moduls.entity.*;
import com.jz.manage.moduls.mapper.*;
import com.jz.manage.moduls.service.FinanceCustomerBalanceService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 企业客户充值记录(TFinanceCustomerBalance)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:39
 */
@Service("financeCustomerBalanceService")
@Transactional
public class FinanceCustomerBalanceServiceImpl implements FinanceCustomerBalanceService {

    private static final Logger logger = LoggerFactory.getLogger(FinanceCustomerBalanceServiceImpl.class);

    @Autowired
    private FinanceCustomerBalanceDao balanceDao;

    @Autowired
    private FinanceCustomerAssetsDao assetsDao;

    @Autowired
    private DepartmentDao departmentDao;

    @Autowired
    private FinanceTradeFlowDao flowDao;

    @Autowired
    private FinanceCashOutDao cashOutDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 充值管理--分页列表
     * @param balanceListRequest
     * @param httpRequest
     * @return
     * @throws Exception
     */
    @Override
    public IPage<BalanceListDto> findListBalance(BalanceListRequest balanceListRequest, HttpServletRequest httpRequest) throws Exception {
        IPage<BalanceListDto> page = new Page<>(balanceListRequest.getPageNum(), balanceListRequest.getPageSize());
        QueryWrapper<BalanceListDto> query = new QueryWrapper();
        if (StringUtils.isNotBlank(balanceListRequest.getSearch())){
            query.and(wrapper -> wrapper.like("t3.department_name",balanceListRequest.getSearch())
                    .or().like("t1.remark",balanceListRequest.getSearch()));
        }
        if (StringUtils.isNotBlank(balanceListRequest.getAuditStatus())){
            query.eq("t1.audit_status",balanceListRequest.getAuditStatus());
        }
        if (StringUtils.isNotBlank(balanceListRequest.getBalanceTime())){
            String startTime = balanceListRequest.getBalanceTime().substring(0, 10)+ " 00:00:00";
            String endTime = balanceListRequest.getBalanceTime().substring(11, 21)+ " 23:59:59";
            query.between( "t1.cre_time",startTime, endTime);
        }
        query.eq("t1.del_flag", "N");
        query.orderByDesc("t1.cre_time");

        IPage<BalanceListDto> list= balanceDao.findListBalance(page, query);
        return list;
    }

    /**
     * 充值管理--充值详情
     * @param customerBalanceId
     * @return
     * @throws Exception
     */
    @Override
    public BalanceInfoDto queryBalanceByid(String customerBalanceId) throws Exception {
        return balanceDao.queryBalanceByid(customerBalanceId);
    }

    /**
     * 充值管理--充值审核
     * @param balanceAuditRequest
     * @return
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result<Object> uptBalanceInfoAuditByid(BalanceAuditRequest balanceAuditRequest) throws ResponseException {
        //1.修改充值记录状态
        FinanceCustomerBalance balance = new FinanceCustomerBalance();
        balance.setRejectReason(balanceAuditRequest.getRejectReason());
        balance.setCustomerBalanceId(Long.valueOf(balanceAuditRequest.getCustomerBalanceId()));
        balance.setUptPerson(getUserId()+"");
        balance.setUptTime(new Date());
        //1.查询出充值金额  和 账户id
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("customer_balance_id", balanceAuditRequest.getCustomerBalanceId());
        wrapper.eq("del_flag", "N");
        FinanceCustomerBalance financeCustomerBalance = balanceDao.selectOne(wrapper);
        Long assetsId = financeCustomerBalance.getAssetsId(); //企业用户资产id
        String balanceMoney = financeCustomerBalance.getBalanceMoney().toString(); //充值金额
        BigDecimal balanceMoneyObj = new BigDecimal(balanceMoney);

        //2.查询资产金额
        QueryWrapper assetsWrapper = new QueryWrapper();
        assetsWrapper.eq("assets_id", assetsId);
        wrapper.eq("del_flag", "N");
        FinanceCustomerAssets financeCustomerAssets = assetsDao.selectOne(assetsWrapper);
        String frozenMoney = financeCustomerAssets.getFrozenMoney().toString(); // 冻结金额
        BigDecimal frozenMoneyObj = new BigDecimal(frozenMoney);
        // 冻结金额大于等于充值金额
        if (frozenMoney.compareTo(balanceMoney) > -1) {
            // 减去冻结金额
            frozenMoneyObj = frozenMoneyObj.subtract(balanceMoneyObj);

            if (AuditStatusEnum.YSH.getValue().equals(balanceAuditRequest.getAuditStatus())) {
                //审核成功 给总金额和可用余额加上充值金额 ，冻结减去
                String useMoney = financeCustomerAssets.getUseMoney().toString(); // 可用金额
                BigDecimal useMoneyObj = new BigDecimal(useMoney);
                useMoneyObj = useMoneyObj.add(balanceMoneyObj);
                //4.更新 企业资产
                FinanceCustomerAssets saveAssets = new FinanceCustomerAssets();
                saveAssets.setAssetsId(assetsId);
                saveAssets.setUseMoney(useMoneyObj);
                saveAssets.setTotalMoney(useMoneyObj);
                saveAssets.setFrozenMoney(frozenMoneyObj);
                saveAssets.setUptTime(new Date());
                saveAssets.setUptPerson(getUserId()+ "");
                if (assetsDao.updateById(saveAssets) != 1) {
                 throw ResponseException.of_error("更新企业资产失败！");
                }
                logger.info("###########################充值审核通过---更新企业资产成功####################################");
                balance.setAuditStatus(balanceAuditRequest.getAuditStatus());
                //添加流水记录
                FinanceTradeFlow financeTradeFlow = new FinanceTradeFlow();
                financeTradeFlow.setCreTime(new Date());
                financeTradeFlow.setTradeMoney(balanceMoneyObj);
                financeTradeFlow.setAssetsId(assetsId);
                financeTradeFlow.setTradeType(TradeFlowEnum.CZ.getValue());
                financeTradeFlow.setTradeFlowNumber(NumberUtils.getDealNum());
                financeTradeFlow.setBalanceMoney(useMoneyObj);
                financeTradeFlow.setDelFlag("N");
//                financeTradeFlow.setCashOutId();
                financeTradeFlow.setCrePerson(getUserId()+ "");
                if (flowDao.insert(financeTradeFlow) != 1) {
                    throw ResponseException.of_error("商城提现流水记录添加失败！");
                }
                logger.info("#####################商城提现流水记录添加成功#####################");
            }
            if (AuditStatusEnum.WTG.getValue().equals(balanceAuditRequest.getAuditStatus())) {
                //4.审核未通过 更新 企业资产
                FinanceCustomerAssets saveAssets = new FinanceCustomerAssets();
                saveAssets.setAssetsId(assetsId);
                saveAssets.setFrozenMoney(frozenMoneyObj);
                saveAssets.setUptTime(new Date());
                saveAssets.setUptPerson(getUserId()+ "");
                if (assetsDao.updateById(saveAssets) != 1) {
                    throw ResponseException.of_error("更新企业资产失败！");
                }
                logger.info("###########################充值审核未通过--更新企业资产成功####################################");
                balance.setAuditStatus(balanceAuditRequest.getAuditStatus());
            }
            if (balanceDao.updateById(balance) != 1) {
                throw ResponseException.of_error("修改充值审核状态失败！");
            }
            logger.info("###########################修改充值审核状态成功####################################");
            return Result.ok();
        }else {
            throw new RuntimeException("提交失败！");
        }
    }

    /**
     * 快捷充值-给用户充值
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result uptAccountMoney(BalanceQuickReq req) throws ResponseException{
        QueryWrapper query = new QueryWrapper();
        query.eq("unified_credit_code", req.getUnifiedCreditCode());
        query.eq("del_flag", "N");
        query.eq("audit_status", "02");
        // 获取充值用户信息
        Department department = departmentDao.selectOne(query);
        if (department.getDepartmentId() != null) {
            // 获取用户资产信息
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("department_id", department.getDepartmentId());
            queryWrapper.eq("del_flag", "N");
            FinanceCustomerAssets assets = assetsDao.selectOne(queryWrapper);
            if (assets.getAssetsId() != null && assets.getUseMoney().compareTo(BigDecimal.ZERO) > -1) {
                // 充值金额
                BigDecimal useMoney = assets.getUseMoney().add(new BigDecimal(req.getBalanceMoney()));
                assets.setUseMoney(useMoney);
                assets.setAssetsId(assets.getAssetsId());
                assets.setTotalMoney(useMoney);
                assets.setUptPerson(getUserId() + "");
                assets.setUptTime(DateUtils.getToday());
                if (assetsDao.updateById(assets) != 1) {
                    throw ResponseException.of_error("充值失败！");
                }
                logger.info("###########################充值通过---更新企业资产成功####################################");
                //充值记录加记录
                FinanceCustomerBalance balance = new FinanceCustomerBalance();
                balance.setAssetsId(assets.getAssetsId());
                balance.setBalanceMoney(new BigDecimal(req.getBalanceMoney()));
                balance.setAuditStatus("02");
                balance.setBalanceTime(DateUtils.getToday());
                balance.setCreTime(DateUtils.getToday());
                balance.setCrePerson(getUserId()+ "");
                balance.setDelFlag("N");
                if (balanceDao.insert(balance) != 1){
                    throw ResponseException.of_error("添加充值记录失败!");
                }
                logger.info("###########################充值通过---添加充值记录成功####################################");
                if (balance.getCustomerBalanceId() != null && StringUtils.isNotEmpty(balance.getAssetsId().toString())) {
                    // 交易流水加记录
                    FinanceTradeFlow tradeFlow = new FinanceTradeFlow();
                    tradeFlow.setAssetsId(assets.getAssetsId());
                    tradeFlow.setCustomerBalanceId(balance.getCustomerBalanceId());
                    tradeFlow.setTradeFlowNumber(NumberUtils.getDealNum());
                    tradeFlow.setTradeMoney(new BigDecimal(req.getBalanceMoney()));
                    tradeFlow.setTradeType("02");
                    tradeFlow.setBalanceMoney(assets.getUseMoney());
                    tradeFlow.setCreTime(DateUtils.getToday());
                    tradeFlow.setCrePerson(getUserId() + "");
                    tradeFlow.setDelFlag("N");
                    if (flowDao.insert(tradeFlow) != 1) {
                        throw ResponseException.of_error("添加交易流水记录失败！");
                    }
                    logger.info("###########################充值通过---添加交易流水记录成功####################################");
                    return Result.of_success("充值成功！");
                }
            }
        }
        return Result.of_error("用户不存在！");
    }

    /**
     * 快捷提现-给用户提现
     * @param req
     * @return
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addCashOutMoney(CashOutQuickReq req) throws ResponseException {
        QueryWrapper query = new QueryWrapper();
        query.eq("unified_credit_code", req.getUnifiedCreditCode());
        query.eq("del_flag", "N");
        query.eq("audit_status", "02");
        // 获取充值用户信息
        Department department = departmentDao.selectOne(query);
        if (department.getDepartmentId() != null) {
            // 获取用户资产信息
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("department_id", department.getDepartmentId());
            queryWrapper.eq("del_flag", "N");
            FinanceCustomerAssets assets = assetsDao.selectOne(queryWrapper);
            if (assets.getAssetsId() != null && assets.getUseMoney().compareTo(BigDecimal.ZERO) > -1) {
//                // 提现金额
                BigDecimal useMoney = assets.getUseMoney().subtract(new BigDecimal(req.getCashOutMoney()));
                assets.setUseMoney(useMoney);
                assets.setAssetsId(assets.getAssetsId());
                assets.setTotalMoney(useMoney);
                assets.setUptPerson(getUserId() + "");
                assets.setUptTime(DateUtils.getToday());
                if (assetsDao.updateById(assets) != 1) {
                    throw ResponseException.of_error("提现失败！");
                }
                logger.info("###########################提现通过---更新企业资产成功####################################");
                //提现记录加记录
                FinanceCashOut cashOut = new FinanceCashOut();
                cashOut.setAssetsId(assets.getAssetsId());
                cashOut.setCashOutMoney(new BigDecimal(req.getCashOutMoney()));
                cashOut.setCashOutTime(DateUtils.getToday());
                cashOut.setAuditStatus("02");
                cashOut.setCrePerson(getUserId() + "");
                cashOut.setCreTime(DateUtils.getToday());
                cashOut.setDelFlag("N");
                if (cashOutDao.insert(cashOut) != 1) {
                    throw ResponseException.of_error("添加提现记录失败！");
                }
                logger.info("###########################提现通过---添加提现记录成功####################################");
                if (cashOut.getCashOutId() != null && cashOut.getAssetsId() != null) {
                    // 交易流水加记录
                    FinanceTradeFlow tradeFlow = new FinanceTradeFlow();
                    tradeFlow.setAssetsId(assets.getAssetsId());
                    tradeFlow.setCashOutId(cashOut.getCashOutId());
                    tradeFlow.setTradeFlowNumber(NumberUtils.getDealNum());
                    tradeFlow.setTradeMoney(new BigDecimal(req.getCashOutMoney()));
                    tradeFlow.setTradeType("01");
                    tradeFlow.setBalanceMoney(assets.getUseMoney());
                    tradeFlow.setCreTime(DateUtils.getToday());
                    tradeFlow.setCrePerson(getUserId() + "");
                    tradeFlow.setDelFlag("N");
                    if (flowDao.insert(tradeFlow) != 1) {
                        throw ResponseException.of_error("添加交易流水记录失败！");
                    }
                    logger.info("###########################提现通过---添加交易流水记录成功####################################");
                    return Result.of_success("提现成功！");
                }
            }
        }
        return Result.of_error("用户不存在！");
    }

    /**
     * 搜索企业
     *
     * @param search
     * @return
     */
    @Override
    public Result searchCompany(String search) {
        // 搜索
        if (StringUtils.isNotEmpty(search)) {
            QueryWrapper<DepartmentAssetsDto> query = new QueryWrapper();
            if (StringUtils.isNotBlank(search)) {
                query.and(wrapper -> wrapper.like("department_name", search)
                        .or().eq("telephone", search)
                        .or().like("linkman", search));
            }
            List<DepartmentAssetsDto> department = departmentDao.selectByDepartment(query);
            return Result.of_success(ResultMsg.SUCCESS, department);
        }
        return Result.of_error("搜索失败!");
    }

    /**
     * 企业交易记录-条件查询
     *
     * @param req
     * @return
     */
    @Override
    public Result tradeCompany(TradeCompanyReq req) {
        IPage<DepartmentTradeDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<DepartmentTradeDto> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getSearch())) {
            queryWrapper.and(s -> s.like("t1.department_name", req.getSearch())
                    .or().like("t1.linkman", req.getSearch()));
        }
        if (StringUtils.isNotBlank(req.getTelephone())) {
            queryWrapper.like("t1.telephone", req.getTelephone());
        }
        queryWrapper.eq("t1.del_flag", DelFlagEnum.NO.getValue());
        queryWrapper.groupBy("t1.department_id");
        queryWrapper.orderByDesc("transactionMoney");
        IPage<DepartmentTradeDto> list = departmentDao.selectByTradeCompany(page, queryWrapper);
        if (list.getRecords() != null) {
            return Result.ok(list);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 企业交易记录-主键查询详情
     *
     * @param departmentId
     * @return
     */
    @Override
    public Result tradeCompanyDetail(Long departmentId) {
        if (departmentId == null) {
           return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        List<TradeCompanyDetailDto> list = departmentDao.tradeCompanyDetail(departmentId);
        if (list == null) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        return Result.ok(list);
    }


    /**
     * 获取当前用户
     * @return
     */
    public Long getUserId() {
        SysUserDto userDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return userDto.getUserId();
    }
}
