package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.config.bean.req.CategoryConfigEditReq;
import com.jz.manage.moduls.controller.config.bean.req.CategoryConfigReq;
import com.jz.manage.moduls.controller.config.bean.req.MenuConfigReq;
import com.jz.manage.moduls.entity.DataGoodsCategory;
import com.jz.manage.moduls.entity.MenuConfig;
import com.jz.manage.moduls.mapper.DataGoodsCategoryDao;
import com.jz.manage.moduls.mapper.MenuConfigDao;
import com.jz.manage.moduls.service.MenuConfigServer;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @ClassName:
 * @Author: Carl
 * @Date: 2021/2/7
 * @Version:
 */
@Service("menuConfigServer")
public class MenuConfigServerImpl implements MenuConfigServer {

    @Autowired
    private MenuConfigDao menuConfigDao;

    @Autowired
    private DataGoodsCategoryDao categoryDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 查询全部菜单
     *
     * @return
     */
    @Override
    public Result findList() {
        QueryWrapper<MenuConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("del_flag", DelFlagEnum.NO.getValue());
        List<MenuConfig> menuConfigs = menuConfigDao.selectList(queryWrapper);
        if (menuConfigs.size() != 0) {
            return Result.ok(menuConfigs);
        }
        return Result.of_error(ResultMsg.DATA_NOT_FOUND);
    }

    /**
     * 编辑菜单
     *
     * @param req
     * @return
     */
    @Override
    public Result editUrl(MenuConfigReq req) {
        MenuConfig menuConfig = new MenuConfig();
        menuConfig.setConfigId(req.getConfigId());
        menuConfig.setConfigUrl(req.getConfigUrl());
        menuConfig.setUptTime(DateUtils.getToday());
        menuConfig.setUptPerson(userDto().getUserId() + "");
        if (menuConfigDao.updateById(menuConfig) != 1) {
            return Result.of_error(ResultMsg.UPDATE_FAIL);
        }
        return Result.ok();
    }

    /**
     * 条件查询分类配置
     *
     * @param req
     * @return
     */
    @Override
    public Result<IPage<DataGoodsCategory>> findCategory(CategoryConfigReq req) {
        IPage<DataGoodsCategory> page = new Page<>(req.getPageNum(),req.getPageSize());

        QueryWrapper<DataGoodsCategory> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(req.getCategoryName())) {
            queryWrapper.like("category_name",req.getCategoryName());
        }
        queryWrapper.eq("parent_id", "0");
        queryWrapper.eq("del_flag",DelFlagEnum.NO.getValue());
        queryWrapper.orderByDesc("cre_time");
        IPage<DataGoodsCategory> list = categoryDao.selectPage(page, queryWrapper);
        if (list.getTotal() != 0) {
            return Result.of_success(list);
        }
        return Result.of_error(ResultMsg.FAILURE);
    }

    /**
     * 编辑分类配置
     *
     * @param req
     * @return
     */
    @Override
    public Result editCategory(CategoryConfigEditReq req) {
        DataGoodsCategory category = new DataGoodsCategory();
        category.setCategoryId(req.getCategoryId());
        category.setCategoryPicture(req.getCategoryPicture());
        category.setCategoryDesc(req.getCategoryDesc());
        category.setUptTime(DateUtils.getToday());
        category.setUptPerson(userDto().getUserId() + "");
        if (categoryDao.updateById(category) != 1) {
            return Result.of_error(ResultMsg.UPDATE_FAIL);
        }
        return Result.ok("分类配置成功！");
    }

    public SysUserDto userDto() {
        SysUserDto userDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return userDto;
    }
}
