package com.jz.manage.moduls.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.HttpsUtilsTest;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.order.bean.dto.ApiInterfaceDetailDto;
import com.jz.manage.moduls.controller.order.bean.dto.OrderDetailDto;
import com.jz.manage.moduls.controller.order.bean.dto.OrderDto;
import com.jz.manage.moduls.controller.order.bean.req.ApiInterfaceDetailReq;
import com.jz.manage.moduls.controller.order.bean.req.OrderRequest;
import com.jz.manage.moduls.controller.order.bean.req.SaltResetReq;
import com.jz.manage.moduls.entity.Order;
import com.jz.manage.moduls.mapper.OrderDao;
import com.jz.manage.moduls.service.OrderService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * 订单表(TOrder)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@Service("orderService")
@Transactional
public class OrderServiceImpl implements OrderService {

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private RedisTemplate redisTemplate;

    @Value("${gateway.gatewayUpSalt}")
    private String gatewayUpSalt;

    @Value("${gateway.getApiDetail}")
    private String getApiDetail;

    /**
     * 订单管理列表（分页查询）
     * @param req
     * @param httpServletRequest
     * @return
     * @throws Exception
     */
    @Override
    public Result<IPage<OrderDto>> queryListOrderInfo(OrderRequest req, HttpServletRequest httpServletRequest) throws Exception {
        IPage<OrderDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<OrderDto> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(req.getSearch())) {
            query.and(wrapper -> wrapper.like("t2.data_name", req.getSearch())
                    .or().like("t4.department_name", req.getSearch()));
        }
        if (StringUtils.isNotBlank(req.getPurchaseMethod())) {
            query.eq("t.purchaseMethod", req.getPurchaseMethod());
        }

        if (StringUtils.isNotBlank(req.getOrderTime())) {
            String startTime = req.getOrderTime().substring(0, 10) + " 00:00:00";
            String endTime = req.getOrderTime().substring(11, 21) + " 23:59:59";
            query.between("t.cre_time", startTime, endTime);

        }
        query.eq("t.del_flag", "N");
        query.orderByDesc("t.cre_time");
        IPage<OrderDto> list = orderDao.queryListOrderInfo(page, query);
        if (list.getTotal() == 0) {
            return Result.of_error("没有订单！");
        }
        return Result.of_success(list);
    }

    /**
     * 根据订单主键查询，订单详情
     * @param orderId
     * @return
     * @throws Exception
     */
    @Override
    public Result queryOrderDetail(String orderId) throws Exception {
        OrderDetailDto orderDetailDto = orderDao.queryOrderDetail(orderId);
        if (orderDetailDto != null) {
            Date endTime = orderDetailDto.getInvalidTime();
            String indate = DateUtils.getTime(endTime, DateUtils.getToday());
            orderDetailDto.setIndate(indate);
        }
        return Result.ok(orderDetailDto);
    }

    /**
     * 重置盐值
     *
     * @param req
     * @return
     */
    @Override
    public Result updateSalt(SaltResetReq req) {
        //调用gatway
        JSONObject object = new JSONObject();
        object.put("authCode", req.getGoodsToken());
        object.put("createUser", getUserId());
        String s = HttpsUtilsTest.submitPost(gatewayUpSalt, object.toString());
        //处理返回的参数
        JSONObject params = JSONObject.parseObject(s);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.error("修改失败！");
        }
        String data = params.getString("data");
        JSONObject dataParam = JSONObject.parseObject(data);
        String salt = dataParam.getString("salt");
        // 修改参数
        Order order = new Order();
        order.setSaltValue(salt);
        order.setOrderId(req.getId());
        order.setUptTime(DateUtils.getToday());
        order.setUptPerson(getUserId() + "");
        if (orderDao.updateById(order) != 1) {
            return Result.of_error(ResultMsg.UPDATE_FAIL);
        }
        Map map = new HashMap();
        map.put("salt", salt);
        return Result.of_success(ResultMsg.SUCCESS, salt);
    }

    /**
     * apikey获取接口文档
     *
     * @param req
     * @return
     */
    @Override
    public Result getOrderApiDetail(ApiInterfaceDetailReq req) {
        //调用gatway
        JSONObject object = new JSONObject();
        object.put("apiKey", req.getApiKey());
        String s = HttpsUtilsTest.submitPost(getApiDetail, object.toString());
        //处理返回的参数
        JSONObject params = JSONObject.parseObject(s);
        if (!params.get("code").equals(200) || org.springframework.util.StringUtils.isEmpty(params.get("data"))) {
            return Result.error("修改失败！");
        }
        String data = params.getString("data");
        JSONObject dataParam = JSONObject.parseObject(data);
        Map<String, Object> map = new HashMap<>();
        ApiInterfaceDetailDto apiInterfaceDetailDto = new ApiInterfaceDetailDto();
        apiInterfaceDetailDto.setApiKey(dataParam.getString("apiKey"));
        apiInterfaceDetailDto.setOutputType(dataParam.getString("outputType"));
        apiInterfaceDetailDto.setTargetUrl(dataParam.getString("targetUrl"));
        apiInterfaceDetailDto.setReqType(dataParam.getString("reqType"));
        map.put("dataInfo", apiInterfaceDetailDto);
        // 响应参数
        List responseParam = returnParams("responseParam", data);
        // 请求参数
        List requestParam = returnParams("requestParam", data);
        // 固定参数
        List inboxParam = returnParams("inboxParam", data);
        map.put("responseParam", responseParam);
        map.put("requestParam", requestParam);
        map.put("inboxParam",inboxParam);
        return Result.ok(map);
    }


    /**
     * 封装参数
     * @param params
     * @param data
     * @return
     */
    public List returnParams(String params, String data) {
        JSONArray list = JSONObject.parseObject(data).getJSONArray(params);
        if (list == null) {
            return null;
        }
        List<ApiInterfaceDetailDto> listParams = new ArrayList<>();
        for (Object o : list) {
            JSONObject result = (JSONObject)o;
            ApiInterfaceDetailDto dto = new ApiInterfaceDetailDto();
            if (StringUtils.isNotEmpty(result.getString("name"))) {
                dto.setParamsName(result.getString("name"));
            }
            if (StringUtils.isNotEmpty(result.getString("type"))) {
                dto.setParamsType(result.getString("type"));
            }
            if (StringUtils.isNotEmpty(result.getString("defaultValue"))) {
                dto.setDefaultValue(result.getString("defaultValue"));
            }
            if (StringUtils.isNotEmpty(result.getString("desc"))) {
                dto.setParamsDesc(result.getString("desc"));
            }
            listParams.add(dto);
        }
        return listParams;
    }
    /**
     * 获取当前用户
     * @return
     */
    public Long getUserId() {
        SysUserDto sysUserDto = (SysUserDto) redisTemplate.opsForValue().get("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);
        return sysUserDto.getUserId();
    }
}