package com.jz.manage.moduls.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.entity.SysUser;
import com.jz.common.utils.Result;
import com.jz.manage.moduls.controller.sys.bean.req.LoginRequest;
import com.jz.manage.moduls.entity.SysMenu;
import com.jz.manage.moduls.mapper.SysUserDao;
import com.jz.manage.moduls.service.SysUserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 平台系统用户表(SysUser)表服务实现类
 *
 * @author Bellamy
 * @since 2020-11-29 14:30:26
 */
@Service("sysUserService")
public class SysUserServiceImpl implements SysUserService {

    @Autowired
    private SysUserDao sysUserDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 登陆
     *
     * @param user
     * @return
     */
    @Override
    public Result loginByUser(LoginRequest user) {
        // 手机登陆
        String ph = "^[1][34578]\\d{9}$";
        Map map = new HashMap();
        if (user.getUsername().matches(ph) && StringUtils.isNotEmpty(user.getUsername())) {
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("telephone", user.getUsername());
            SysUser sysUser = sysUserDao.selectOne(queryWrapper);
            if (sysUser.getUserId() == null) {
                return Result.of_error("用户不存在，请注册！");
            }
            if (sysUser.getPassword().equals(sysUser.getPassword())) {
                SysUserDto sysUserDto = new SysUserDto();
                BeanUtils.copyProperties(sysUser,sysUserDto);
                redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS, sysUserDto, 3, TimeUnit.DAYS);
                return Result.of_success(ResultMsg.LOGIN_SUCCESS, sysUser);
            }
            return Result.of_error(ResultMsg.LOGIN_ERROR);
        }
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("account", user.getUsername());
        SysUser sysUser = sysUserDao.selectOne(queryWrapper);
        if (sysUser == null) {
            return Result.of_error("用户不存在，请注册！");
        }
        if (sysUser.getPassword().equals(sysUser.getPassword())) {
            SysUserDto sysUserDto = new SysUserDto();
            BeanUtils.copyProperties(sysUser,sysUserDto);
            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS, sysUserDto, 3, TimeUnit.DAYS);
            return Result.of_success(ResultMsg.LOGIN_SUCCESS, sysUser);
        }
        return Result.of_error(ResultMsg.LOGIN_ERROR);
    }


    @Override
    public Map<String, Object> getUserRoleByAccount(Long userId, Long roleId) {
        Set<String> roleSet = new HashSet<>();
        Set<String> roleMenuSet = new HashSet<>();
        //查询用户拥有的角色
        List<Map> list = sysUserDao.getUserRoleByAccount(userId, roleId);
        if (list.size() > 0 && list != null) {
            for (Map map : list) {
                roleSet.add(map.get("roleName").toString());
                roleSet.add(map.get("roleId").toString());
                roleMenuSet.add(map.get("menuId").toString());
                roleMenuSet.add(map.get("menuName").toString());
                roleMenuSet.add(map.get("parentId").toString());
                roleMenuSet.add(map.get("url").toString());
            }
        }
        Map<String, Object> userInfo = new HashMap<>();
        userInfo.put("roleSet", roleSet);
        userInfo.put("roleMenuSet", roleMenuSet);
        return userInfo;
    }

    /**
     * 平台--根据角色展示菜单
     * @param userId
     * @param roleId
     * @return
     */
    @Override
    public Result getMenu(Long userId, Long roleId) {

        //查询用户拥有的角色
        List<SysMenu> entities = sysUserDao.getMenu(userId, roleId);

        // 找到所有的一级分类
        List<SysMenu> level1Menus = entities.stream().filter(sysMenuEntity ->
                sysMenuEntity.getParentId() == 0 && sysMenuEntity.getIfChild().equals("N")
        ).map((menu)->{
            menu.setChildren(getChildrens(menu,entities));
            return menu;
        }).collect(Collectors.toList());
        return Result.of_success(ResultMsg.SUCCESS, level1Menus);
    }


    private List<SysMenu> getChildrens(SysMenu sysMenu,List<SysMenu> all) {

        List<SysMenu> children = all.stream().filter(sysMenuEntity -> {
            return sysMenuEntity.getParentId() == sysMenu.getMenuId() && sysMenuEntity.getIfChild().equals("Y");
        }).map(sysMenuEntity -> {
            //1、找到子菜单,同时递归调用
            sysMenuEntity.setChildren(getChildrens(sysMenuEntity, all));
            return sysMenuEntity;
        }).collect(Collectors.toList());
        return children;
    }

}